################################################################################
#                                                                              #
# Function that implements the procedure described by Laeuter to evaluate      #
# the correlation of correlated variable groups to a set of seed variables in  #
# a large microarray data set.                                                 #
# The Westfall & Young resampling strategy is used in order to obtain          #
# multiplicity corrected P-values for all variables.                           #
#                                                                              #
################################################################################
#                                                                              #
# Parameters:                                                                  #
#                                                                              #
# data - a data matrix, a data.frame, an AffyBatch-object, an exprsSet-object  #
#        or and ExpressionSet-object containing the complete data.             #
#                                                                              #
# classes - a 'factor'-variable or a list of sample sizes to determine the     #
#           affiliation of the samples. If 'classes' is not set the data are   #
#           considered to form a single sample group.                          #
#                                                                              #
# grpdata - this parameter is not NULL only if the procedure should test only  #
#           specified groups of variables and not search for significant       #
#           groups among all possible groups. 'grpdata' can be:                #
#           a) a matrix, data frame with two columns. In the first column      #
#              contains group names (pivot gene ID's) repeated as many times   #
#              as there are group members. It is the format of .grpdata file   #
#           b) a list of groups where the members are given by their indices   #
#              in the data matrix, by the names of the corresponding columns   #
#              in the data matrix or gene names if the data is an ExprSet.     #
#              The elements of the list are named. Usually the names are the   #
#              gene ID's of the pivot genes.                                   #
#                                                                              #
# seed - the vector of seed variables as character strings, column indices or  #
#       a factor with two levels.The second will be substracted from the first #
#                                                                              #
# incl.seed - a logical variable. If FALSE (default) then the seed variables   #
#             are excluded from the pool of variables which are used for       #
#             creating clusters of correlated variables. If TRUE then the      #
#             seed variables can be included in the clusters. The latter       #
#             may be regarded as not entirely correct from the statistical     #
#             point of view.                                                   #
# corrtype - obsolete. Set to PEARSON. To be removed                           #
#                                                                              #
# r2min - a threshold for the square of the correlation coefficient between    #
#         a pivot variable and the variables in its variable set. Only         #
#         exceeding this threshold are included in the variable set.           #
#                                                                              #
# rposonly - a boolean switch. If 'rposonly' is 'TRUE' only positive           #
#            correlation between a pivot variable and the variables in its     #
#            set is allowed. This condition is checked while creating the      #
#            variable sets.                                                    #
#                                                                              #
# direction - decides whether one-sided or two-sided tests are performed.      #
#             If "positive" then only positive significances are allowed i.e., #
#             clusters with pivot variables negatively correlated with         #
#             the target variable get 0 as the value of their test statistic.  #
#             If "negative" then only negative significances are allowed.      #
#             If "both" then a two-sided test is performed, i.e. both types of #
#             significances, negative and positive are considered.             # 
#                                                                              #
# pivdom - If TRUE then the variance of the pivot variable must be greater     #
#          then the variance of other member variables of the set. This        #
#          condition is checked while creating variable sets and prevents      #
#          creation of identical variable sets.                                #
# discret - a boolean swith. If TRUE then the truncated univariate statistics  #
#           i.e. smaller than 'ucrit' or 'minbeta' will be set to 0 and the    # 
#           untruncated will be set to 1.                                      #
#                                                                              #
# trunc - a boolean switch. If TRUE then the the univariate statistics will    #
#         be truncated.                                                        #
#                                                                              #
# ucrit - a critical threshold for a univariate B-statistics. Statistics       #
#         smaller then that threshold will be truncated (set to 0)             #
#                                                                              # 
# minbeta - another truncation threshold. Univariate B-statistics smaller      #
#           then 'minbeta' will be truncated, i.e., set to 0.                  #
#                                                                              #
# samedirect - a boolean switch. If TRUE than the univariate B-statistics      #
#              which do not correlate with the seed variable with the same     #
#              sign as the pivot variable will be truncated.                   #
#                                                                              #
# maxgene - number of greatest univariate statistics which will be used in     #
#           the computation of the overall statistic of a gene set.            #
#                                                                              #
# r2minseed - lower threshold for the squared correlation between a variable   #
#             from the seed variables and other seed variables. Seed variables #
#             correlating above the threshold with a seed variable will form   #
#             a subset of the seed variables which may be used to compute      #
#             a principal component used later as a summary for the seed       #
#             variables. Which of the subsets of seed variables will be chosen #
#             is decided based on the sum of variances of the variables in     #
#             a set and the nuber of variables in the set                      #
#                                                                              #
# rposonlyseed - If TRUE then only positive correlations are allowed in a set  #
#                of seed variables.                                            #
#                                                                              # 
# nres - the desired number of resamplings.                                    #
#                                                                              #
# rot - a switch that controls whether resampling is done with rotations or    #
#       with permutations. The parameters rot and ranks cannot be both TRUE    #
#                                                                              #
# rand - a switch that controls the randomness of the procedure. If rand is    #
#        set to 'FALSE' (default) repetitions of the same analysis will        #
#        produce the same results.                                             #
#                                                                              #
# multisample - a switch indicating whether permutation should be restricted   #
#               to sample groups (Default: TRUE).                              #
#                                                                              #
# quiet -                                                                      #
#                                                                              #
# thresholdfun - a character string determining a function to constrain        #
#                the variable space.                                           #
#                                                                              #
# threshold - a list with the minimum and maximum threshold for the threshold  #
#             function. Variables which are not in the range will be filtered  #
#             out at the begining.                                             #
#                                                                              #
# ranks - an indicator whether ranks shoud be used or not (for more robustness)#
#                                                                              #
# file - the name of an output file (Default: stdout).                         #
#                                                                              #
# p.details - up to which p-value should the detailed results be saved.        #
#                                                                              #
################################################################################
#                                                                              #
# Return value:                                                                #
#                                                                              #
# An ordered list (ordered by increasing P-value) of all tested variables.     #
#                                                                              #
################################################################################

"SETGEN" <- 
structure(function(x, grpdata=NULL, y, resp.type=c("Quantitative", 
                   "Two class unpaired"), 
                   r2min=0.5, rposonly=TRUE, 
                   direction=c("both","positive","negative"), pivdom=TRUE,
                   transform.beta=c("none","binary","truncated"), minbeta=0.0,
                   samedirect=FALSE, maxgene=NULL,
                   adj.method=c("Single step", "Step down"),
                   nres=100, rot=FALSE, rand=FALSE,  
                   quiet=TRUE, thresholdfun=NULL, threshold=list(NULL,NULL), 
                   ranks=FALSE,
                   file="", p.details=0.05, 
                   crit.vals=sort(unique(c(0.1,0.05,0.01,0.001,p.details)),
                                  decreasing=TRUE)) {

  #############################################################################
  #                                                                           #
  # Some redefinitions of the parameters. They are intended to make.          #
  # the interface more user-friendly.                                         #
  #                                                                           #
  #############################################################################

  data <- x
  rm(x)
  seed <- y
  rm(y)
  resp.type <- match.arg(resp.type)
  classes <- NULL
  incl.seed <- FALSE
  corrtype <- "PEARSON"
  ucrit <- 0.0
  #corrtype <- match.arg(corrtype)
  transform.beta <- match.arg(transform.beta)
  discret <- FALSE
  trunc <- FALSE
  if(transform.beta == "binary") discret <- TRUE
  if(transform.beta == "truncated") trunc <- TRUE
  r2minseed <- 0.25
  rposonlyseed <- TRUE
  multisample <- FALSE
  adj.method <- match.arg(adj.method)
  if(adj.method == "Single step") singlestep <- TRUE
  if(adj.method == "Step down")   singlestep <- FALSE

  #############################################################################
  # 
  # Checking the validity of the parameters
  #
  #############################################################################

  direction <- match.arg(direction)
  direction <- 
       ifelse(direction == "positive", 1, ifelse(direction == "negative", -1, 0))
  if (direction != 0 && rposonly != TRUE)
      stop(paste(" if direction is not equal to 0, which means that a one-sided",
                 "test should be performed than variables in a cluster must be ",
                 "positively correlated with their pivot variable. Therefore,",
                 "rposonly must be TRUE"))
  if (ranks && rot) stop(" rotations and ranks are not compatible ")
  if (nres==0 && rot) 
     stop(" nres = 0 indicates systematic permutations, so rot cannot be TRUE ")
  if (!is.null(grpdata) && !is.null(thresholdfun)) {
      warning(paste("setting the threshold function is not compatible with",
              "defining groups of variables to be analysed. All submitted",
              "groups will be analysed."))
      thresholdfun <- NULL
      threshold <- list(NULL,NULL)
  }
  if((discret == TRUE) & (trunc == TRUE))
     stop("the statistic can be either discretized or truncated, but not both")
  if (is.null(maxgene) || maxgene == FALSE) maxgene <- 0
  if (maxgene != as.integer(maxgene) || maxgene < 0) 
      stop("incorrect maxgene value")
  if (!is.logical(samedirect))
      stop("the parameter samedirect should be either TRUE or FALSE")
  #if (r2min > 1) { 
  #    r2min <- 1
  #    warning("r2min was greater than 1. r2min = 1 will be used.")
  #}

  ##############################################################################
  #                                                                            #
  # Check data type of 'data'. Get the data dimensions                         #
  #                                                                            #
  ##############################################################################

  X <- data.mat(data)
  n <- nrow(X)
  p <- ncol(X)  

  ##############################################################################
  #                                                                            #
  # Identify and check the seed variables                                      #
  #                                                                            #
  ##############################################################################

  #nseed <- length(seed)
  #if (nseed<1) stop("no seed variable(s) specified")
  #X.colnames <- colnames(X)
  #if (is.character(seed)) {
  #  if (length(intersect(seed,X.colnames))<nseed)
  #    stop("unknown variable name(s) in seed set")
  #  seed <- sapply(seed,function(name) which(name==X.colnames))
  #} else if (is.numeric(seed)) {
  #  if (sum(seed>p)>0 || sum(seed<1)>0)
  #    stop("unknown variable indices in seed set")
  #} else if (is.factor(seed)) {
  #  seed <- factor(seed)
  #  if(length(seed) != n)
  #     stop("seed is a factor length(seed) != number of individuals")
  #  if(nlevels(seed) != 2)
  #     stop("if seed is a factor then it should have 2 levels")
  #  seed <- (3 - as.numeric(seed) * 2)
  #  X <- cbind(X,seed)
  #  p <- p + 1
  #  colnames(X)[p] <- "binary.seed" 
  #  seed <- p
  #  nseed <- 1 
  #} else stop("'seed' has to be of types 'character' or 'numeric'")

  if(length(seed) != n) 
     stop("y is not equal to the number of rows of x")
  if(resp.type == "Quantitative") {
     if(!is.numeric(seed)) stop("y is not a quantitative response variable")
     X <- cbind(X, seed)
     p <- p + 1
     colnames(X)[p] <- "quantitative.response"
     seed <- p
     nseed <- 1
  }
  if(resp.type == "Two class unpaired") {
    seed <- factor(seed)
    if(nlevels(seed) != 2)
       stop("y should have 2 levels if resp.type is Two class unpaired")
    seed <- (3 - as.numeric(seed) * 2)
    X <- cbind(X,seed)
    p <- p + 1
    colnames(X)[p] <- "two.class.unpaired.response" 
    seed <- p
    nseed <- 1 
  }

  ##############################################################################
  #                                                                            # 
  # Check 'grpdata' if given and extract the relevant data matrix              #
  # find indexes 'grpidx' of the variable group members                        #
  # in this extracted data matrix. Add the seed variables.                     #
  # incl.seed is set to FALSE so that the seed variables are removed later     #
  # from the data when seed variables are processed. Then                      #
  # only the variables which belong to the prespecified variable groups        #
  # remain in X and the principal component of the seed is added at the end    #
  # Note that this procedure is correct also if seed genes are present in      #
  # the prespecified groups.                                                   #
  #  Update the data size.                                                     #
  #                                                                            #
  ##############################################################################

  if (!is.null(grpdata)) {
      if (is.data.frame(grpdata)) {
          pg <- process.grpdata(grpdata)
      } else { 
          if (!is.list(grpdata))
              stop("grpdata should be a list or a data.frame")
          if(is.null(grpdata$glen) || is.null(grpdata$gnum) || 
             is.null(grpdata$idx)) {
             if (is.null(names(grpdata)))
                 stop(paste("if grpdata is a list then its elements (sets)", 
                            "should be named with the names of the source",
                            "variables of the sets"))
             pg <- list(grpdata = grpdata,
                        glen = sapply(grpdata, length),
                        gnum = length(grpdata),
                        idx = unlist(grpdata),
                        gnames = names(grpdata))
          } else {
             pg <- grpdata
          }
        
      }
      glen <- pg$glen
      gnum <- pg$gnum
      #if (is.integer(pg$idx)) {
      #    if(any(pg$idx)<0) stop("invalid variable index")
      #    if(any(pg$idx)>ncol(X)) stop("invalid variable index")
      #} else 
      if (is.character(pg$idx)) {
          X.colnames <- colnames(X)
          if (is.null(X.colnames)) stop("'data' contains no variable names")
          tmpidx <- match(pg$idx, X.colnames)
          if (any(is.na(pg$idx))) 
              stop(paste("variable name(s) not found:", pg$idx[is.na(tmpidx)]))
          pg$idx <- tmpidx        
      } else stop(" variables have to be represented by names") #or indices")     
      
      tmpidx <- unique(pg$idx)
      X <- X[,c(tmpidx,seed), drop=FALSE]
      grpidx <- match(pg$idx, tmpidx)
      incl.seed <- FALSE
      seed <- length(tmpidx) + (1:length(seed))
      p <- ncol(X)
  } 

  ##############################################################################
  #                                                                            #
  # Subtract the means. If required, compute and save                          # 
  # variation coefficients *before* substracting the means.                    #
  #                                                                            #
  ##############################################################################

  # if required compute and save variation coefficients because later   #
  # they will be impossible to compute, since means will be substracted #
  if (!is.null(thresholdfun)) {
      if (!is.list(thresholdfun))
          thresholdfun <- list(thresholdfun)
      if (any(sapply(thresholdfun, 
                     function(x) attributes(x)$name) == "varcoefficient"))
          varcoeff <- apply(X, cols(), varcoefficient)
  }

  X <- sweep(X, cols(), apply(X, 2, mean))   ###(t(X)-apply(X,cols(),mean))

  ##############################################################################
  #                                                                            #
  # Check number of classes by investigating the 'classes' variable.           #
  #                                                                            #
  ##############################################################################
  
  pc <- process.classes(n,classes)
  s <- pc$s
  ns <- pc$ns
  X <- X[pc$idx,]

  ##############################################################################
  #                                                                            #
  # Check 'multisample'. 'multisample' can be 'TRUE' only if the data consist  #
  # of more than one sample group.                                             #
  #                                                                            #
  ##############################################################################

  if (s==1) multisample <- FALSE

  ##############################################################################
  #                                                                            #
  #  compute principal component of the best correlating seed variables and    #
  #  move the PC of the seed to the end of 'X'.                                #
  #                                                                            #
  ##############################################################################

  if (nseed>1) {
    Xseed <- X[,seed]
    coridx <- lapply(1:nseed,function(idx) {
      tempidx <- which(cor(Xseed,Xseed[,idx])^2>=r2minseed);
      if (rposonlyseed) {
        tempidx <- tempidx[cor(Xseed[,tempidx],Xseed[,idx])>0]
      }
      tempidx
    })
    coridxlen <- unlist(lapply(coridx,length))
    corsum <- unlist(lapply(1:nseed,function(idx)
      sum(cor(Xseed[,coridx[[idx]]],Xseed[,idx])^2)))
    if (max(coridxlen)==1) {
      Xseed.colnames <- colnames(Xseed)
    }
    Xseed <- as.matrix(Xseed[,coridx[[order(coridxlen,corsum,
                                            decreasing=TRUE)[1]]]])
    if (max(coridxlen)==1) {
      colnames(Xseed) <- Xseed.colnames[coridx[[order(coridxlen,corsum,
                                                      decreasing=TRUE)[1]]]]
    }
    if (ncol(Xseed)>1) {
      # PEARSON correlation here correct???
      Xseed.cor <- cor(Xseed)
      Xseed.diag <- diag(sqrt(1/diag(t(Xseed) %*% Xseed)))
      d <- eigen(Xseed.cor)$vectors[,1]
      maxsumidx <- which.max(apply((Xseed.diag%*% (t(Xseed) %*% Xseed) 
                                       %*% Xseed.diag)^2,2,sum))
      if (d[maxsumidx] < 0) d <- -d
      princomp.Xseed <- Xseed %*% Xseed.diag %*% d
    } else { 
      d <- 1 
      maxsumidx <- 1
      princomp.Xseed <- Xseed 
    }
  } else {
    d <- 1
    maxsumidx <- 1
    Xseed <- X[,seed, drop=FALSE]
    princomp.Xseed <- Xseed
  }

  if (incl.seed) {
        X <- cbind(X, princomp.Xseed)
  } else {
        X <- cbind(X[,setdiff(1:p,seed)], princomp.Xseed)
  }
  p <- ncol(X)
  
  colnames(X)[ncol(X)] <- "seed"
  X.colnames <- colnames(X)
  
  ##############################################################################
  #                                                                            #
  # Apply threshold function if specified.                                     #
  #                                                                            #
  ##############################################################################

  if (!is.null(thresholdfun)) {

    if (!is.list(thresholdfun))
         thresholdfun <- list(thresholdfun)
    if (!is.list(threshold))
        stop("threshold must be a list")
   
    if (length(thresholdfun) == 1) 
        if (!all(sapply(threshold, is.list)))
            threshold <- list(threshold)
        else if (!any(sapply(threshold, is.list)))
            stop("something is wrong with the threshold parameter") 
     
    if (length(thresholdfun) > 1) 
        if(!any(sapply(threshold, is.list)) || any(sapply(threshold, length) != 2))
        stop(paste("every item in the list of thresholds must be a list",
                   "containing the lower and upper limit"))
    if (length(threshold) != length(thresholdfun))
        stop("the length of threshold differs from the length of thresholdfun")
 
    X.t <- X[,1:(p-1)]

    for(i in 1:length(thresholdfun)) {
        if (!("name" %in% names(attributes(thresholdfun[[i]])))) 
            attr(thresholdfun[[i]],"name") <- "unknown" 
    }
    
    thresholdidx <- list()
    for( i in 1:length(thresholdfun)) {
         if (attributes(thresholdfun[[i]])$name == "varcoefficient") {
             if (incl.seed) 
                 thfun.val <- varcoeff
             else
                 thfun.val <- varcoeff[-seed]
         } else {
             thfun.val <- apply(X.t, cols(), thresholdfun[[i]])
         }          

         if ( !is.null(threshold[[i]][[1]]) )
              lowerlimit.passed <- thfun.val >= threshold[[i]][[1]]
         else 
              lowerlimit.passed <- rep(TRUE, length(thfun.val))

         if ( !is.null(threshold[[i]][[2]]) )
              upperlimit.passed <- thfun.val <= threshold[[i]][[2]] 
         else
              upperlimit.passed <- rep(TRUE, length(thfun.val))
                 
         thresholdidx[[i]] <- lowerlimit.passed & upperlimit.passed
         if (sum(thresholdidx[[i]])==0) 
             stop("no variable satisfies ",threshold," for '",
             attr(thresholdfun[[i]],"name"),"'")
    }
    
    # elementwise logical AND of all vectors which  #
    # are the elements of the list thresholdidx     #
    thresholdidx <- do.call("rbind", thresholdidx)
    thresholdidx <- apply(thresholdidx, 2, all)

    if (sum(thresholdidx)==0) {
      stop("no variable satisfies all thresholds")
    } else { 
        X.t <- X.t[,thresholdidx, drop=FALSE]
    }
    pt <- ncol(X.t)

    v.name=X.colnames[p]
    X <- cbind(X.t,X[,p])
    colnames(X)[pt+1] <- v.name
  }

  ##############################################################################
  #                                                                            #
  # Apply ranking if desired (that replaces the original data values by their  #
  # rank).                                                                     #
  #                                                                            #
  ##############################################################################

  if (ranks) {
    X <- apply(X,cols(),rank)
    X <- sweep(X, cols(), apply(X, 2, mean))  ### IS IT CORRECT ??????????????
  }

  ##############################################################################
  #                                                                            #
  # Create output files.                                                       #
  #                                                                            #
  ##############################################################################

  if (file=="") {
    summary.file  <- ""
    overview.file <- ""
    details.file  <- ""
    grpdata.file  <- ""
  } else {
    summary.file  <- paste(file,"summary",sep=".")
    overview.file <- paste(file,"overview",sep=".")
    details.file  <- paste(file,"details",sep=".")
    grpdata.file  <- paste(file,"grpdata",sep=".")
    #if (summary.file %in% dir()) system(paste("rm",summary.file))
    #if (overview.file %in% dir()) system(paste("rm",overview.file))
    #if (details.file %in% dir()) system(paste("rm",details.file))
    #if (grpdata.file %in% dir()) system(paste("rm",grpdata.file))
  }

  ##############################################################################
  #                                                                            #
  # Write parameters into a file or onto screen.                               #
  #                                                                            #
  ##############################################################################

    sum.list <- list(list(tag="Type of response",
                          value=resp.type,
                          level=1),
                     list(tag="Data set information",level=1),
                     list(tag="Number of Samples (n)",value=as.character(n),
                          level=2))
    sum.list[[length(sum.list)+1]] <- 
                     list(tag="Number of variables (p)",value=as.character(p-1),
                          level=2)

    if (!is.null(thresholdfun)) {
      sum.list[[length(sum.list)+1]] <-
                     list(tag="Threshold function usage",level=1)
      for (i in 1:length(thresholdfun)) {
        sum.list[[length(sum.list)+1]] <-
                       list(tag="Threshold function",
                            value=attr(thresholdfun[[i]],"name"), level=2)
        sum.list[[length(sum.list)+1]] <-
                       list(tag="Threshold (lower,upper)",
                            value=paste("(",threshold[[i]][[1]],
                                        ", ",threshold[[i]][[2]], ")", sep=""))
      }
      sum.list[[length(sum.list)+1]] <-
                     list(tag="New number of variables (p)",
                          value=as.character(pt))
      p <- pt + 1
    }

    sum.list[[length(sum.list)+1]] <-
                     list(tag="Calculation with ranks",value={if(ranks)
                          "yes" else "no"},level=1)
    sum.list[[length(sum.list)+1]] <-
                     list(tag="Resampling information",level=1)
    if (!rot && nres>0)
    sum.list[[length(sum.list)+1]] <-
                     list(tag="Type of resampling",
                          value="Random permutations",level=2)
    if (nres==0)
      sum.list[[length(sum.list)+1]] <-
                     list(tag="Type of resampling",
                          value="Systematic permutations",level=2)
    if (rot)
      sum.list[[length(sum.list)+1]] <-
                     list(tag="Type of resampling",
                          value="Random rotations",level=2)
    if (s>1 && multisample) 
      sum.list[[length(sum.list)+1]] <-
                     list(tag="Multiple permutation classes",
                          value="yes")
      
    sum.list[[length(sum.list)+1]] <-
                     list(tag="Randomized resampling",
                          value={if(rand) "yes" else "no"})
    sum.list[[length(sum.list)+1]] <-
                     list(tag="Number of resamplings",
                          value=as.character(ifelse(nres==0, factorial(nrow(X)), 
                          nres)))
    
    sum.list[[length(sum.list)+1]] <-
                     list(tag="Definition of gene sets",level=1)
    
    sum.list[[length(sum.list)+1]] <-
                     list(tag="Gene sets were",
                          value={if(is.null(grpdata)) 
                                    "created based on correlations"
                                 else "defined a priori"},level=2)
    if(is.null(grpdata)) {
        sum.list[[length(sum.list)+1]] <-
                         list(tag="Minimum squared correlation",
                              value=as.character(r2min),level=2)
        if (rposonly) {
          sum.list[[length(sum.list)+1]] <-
                         list(tag="Direction of correlation",value="Positive",
                              level=2)
        } else {
          sum.list[[length(sum.list)+1]] <-
                         list(tag="Direction of correlation",
                              value="Positive/negative",level=2)
        }

        if (direction==0)
          sum.list[[length(sum.list)+1]] <-
                         list(tag="Direction of significance",
                              value="Positive/negative (two-sided test)",
                              level=2)
        if (direction==-1)
          sum.list[[length(sum.list)+1]] <-
                         list(tag="Direction of significance",
                              value="Negative (one-sided test)",level=2)
        if (direction==1)
          sum.list[[length(sum.list)+1]] <-
                         list(tag="Direction of significance",
                              value="Positive (one-sided test)",level=2)

        sum.list[[length(sum.list)+1]] <-
                         list(tag="Dominance of pivot variable",
                              value={if (pivdom) "yes" else "no"},level=2)
    }
   

    sum.list[[length(sum.list)+1]] <-
                     list(tag="Test function",value="Sum of Beta statistics",
                          level=1)
    sum.list[[length(sum.list)+1]] <-
                     list(tag="Truncation/discretization",value={if (trunc) 
                          "truncation" else {if(discret) "discretization" 
                          else "no"}},
                          level=2)
    if (trunc || discret)
    sum.list[[length(sum.list)+1]] <-
                     list(tag="Truncation/discretization threshold",
                          value=minbeta, level=2)
    sum.list[[length(sum.list)+1]] <-
                     list(tag="Contribution from the highest statistics only",
                          value={if(maxgene == 0) "no" else maxgene}, level=2)

    sum.list[[length(sum.list)+1]] <-
                     list(tag="Multiple testing adjustment",
                          value=adj.method,
                          level=1)

  if (!quiet) {
    SETGEN.print.parameters(sum.list,file=summary.file)
  }

  ##############################################################################
  #                                                                            #
  # Process resampling.                                                        #
  # The parameters are a little different for rotations.                       #
  # If 'grpdata' is not given then vectors: of indices of the variables which  #
  # make up the groups 'grpidx',                                               #
  # group lengths 'glen' and the number of groups 'gnum' are filled            # 
  # with zeros and passed to the function.                                     #
  # If 'grpdata' is not given then the number of created genesets is           #
  # 'clustnum'=p-1. i.e., number of variables excluding the seed at the end.   #
  # If 'grpdata' is given then the number of created genesets is               #
  # 'clustnum'= 'gnum', i.e., the prescpecified number of variable groups.     #
  # 'pivotidx' is a vector of pivot variable indices.                          #
  #  If it 'grpdata' is not given then 'pivotidx' is set to c(0,0), because    #
  #  Then each gene one after another will be taken as a pivot variable and    #
  #  therefore the index of the current pivot gene will be known.              #
  #                                                                            #
  ##############################################################################

  if (is.null(grpdata)) {
      grpidx <- c(0,0)
      glen <- c(0,0)
      gnum <- 0
      clustnum <- p-1
      pivotidx <- c(0,0) 
  } else {
      clustnum <- gnum
      pivotidx <- match(pg$gnames, colnames(X))
      if (is.na(any(pivotidx))) 
          stop(paste("pivot genes not found in the data:", 
                      pg$gnames[is.na(pivotidx)]))
      if (length(pivotidx) != gnum) stop("# pivot variables != # groups")
  } 

  if (corrtype=="PEARSON") {
    if (rot) {
     res <- .C("wy_corr_laeuter_pearson_R",
               as.double(crossprod(helmert(n),X)), as.integer(n-1), as.integer(s), 
               as.integer(p), 
               as.integer(grpidx), as.integer(glen), as.integer(gnum), 
               as.integer(pivotidx),
               as.integer(rposonly),as.integer(direction),as.integer(pivdom),
               as.double(r2min),
               as.integer(nres),as.integer(rand),as.integer(multisample), 
               as.integer(rot),
               as.integer(discret), as.integer(trunc), as.double(ucrit),
               as.double(minbeta), as.integer(samedirect), as.integer(maxgene),
               as.integer(singlestep),
               idx=integer(clustnum), testval=double(clustnum), 
               pval=double(clustnum),
               critlevs=as.double(crit.vals),ncrit=as.integer(length(crit.vals)),
               critvals=double(length(crit.vals)))
    } else
     res <- .C("wy_corr_laeuter_pearson_R",
               as.double(X), as.integer(ns), as.integer(s), as.integer(p),
               as.integer(grpidx), as.integer(glen), as.integer(gnum),
               as.integer(pivotidx),
               as.integer(rposonly), as.integer(direction), as.integer(pivdom),
               as.double(r2min),
               as.integer(nres),as.integer(rand),as.integer(multisample), 
               as.integer(rot),
               as.integer(discret), as.integer(trunc), as.double(ucrit),
               as.double(minbeta), as.integer(samedirect), as.integer(maxgene),
               as.integer(singlestep),
               idx=integer(clustnum), testval=double(clustnum), 
               pval=double(clustnum),
               critlevs=as.double(crit.vals),ncrit=as.integer(length(crit.vals)),
               critvals=double(length(crit.vals)))

  } else {
    res <- .C("wy_corr_laeuter_total_R",
              as.double(X), as.integer(ns), as.integer(s), as.integer(p),
              as.integer(grpidx), as.integer(glen), as.integer(gnum),
              as.integer(rposonly),as.integer(pivdom),as.double(r2min),
              as.integer(nres),as.integer(rand),as.integer(multisample),
              idx=integer(clustnum), testval=double(clustnum), 
              pval=double(clustnum),
              critlevs=as.double(crit.vals),ncrit=as.integer(length(crit.vals)),
	      critvals=double(length(crit.vals)))
  }

  ##############################################################################
  #                                                                            #
  # Print results to screen or write it into the specified file.               #
  #                                                                            #
  ##############################################################################
 
  ##############################################################################
  #                                                                            #
  # Get indices (pivot variables) of most relevant clusters.                   #
  # If no cluster is significant then get ten clusters with smallest p-values. #
  #                                                                            #
  ##############################################################################
  
  detidx <- rev(which(res$pval<=p.details))
  if (!length(detidx)) {
    if (clustnum >= 10) {
      detidx <- -(0:9)+clustnum
    } else {
      detidx <- clustnum:1
    }
  }
  pivots <- res$idx[detidx]

  ##############################################################################
  #                                                                            #
  # Get relevant clusters. If 'grpdata' is not given build clusters by adding  #
  # variables to each pivot variable. The added variables have to satisfy      #
  # the correlation and other thresholds. If 'grpdata' is given then just take #
  # the given groups. If r2min is very close to 1 (precision up to 0.00001)    #
  # then it is set to exactly 1. The purpose is to force analysis on single    #
  # genes.                                                                     #
  #                                                                            #
  ##############################################################################

  if (is.null(grpdata)) {

    detclust <- sapply(pivots,function(pivot) {
      ##if(abs(r2min - 1) < 0.00001) { 
      ##    r2min <- 1
      ##    tempidx <- which(cor(X[,pivot],X[,1:(p-1)])^2>=(r2min-0.00001))
      ##} else {
          #tempidx <- which(cor(X[,pivot],X[,1:(p-1)])^2>=r2min)
      tempidx <- unique(c(pivot, which(cor(X[,pivot],X[,1:(p-1)])^2>=r2min)))
      ##}
      if (rposonly) {
        tempidx <- tempidx[cor(X[,pivot],X[,tempidx])>0]
      }
      if (pivdom) {
        tempidx <- tempidx[apply(as.matrix(X[,tempidx]),cols(),sqsum)<=
                           sqsum(X[,pivot])]
      }
      tempidx
    })
    if (length(pivots)==1) detclust <- list(detclust)   
    detclust <- lapply(detclust,function(clustidx) colnames(X)[clustidx])
    names(detclust) <- colnames(X)[pivots]
  } else {
    
    detclust <- pg$grpdata[pivots] 
  }

  ##############################################################################
  #                                                                            #
  # Output the relevant clusters either to a file or to the screen.            #
  #                                                                            #
  ############################################################################## 

  if (is.numeric(detclust[[1]])) {
  mats <- lapply(detclust,function(clust) { X.clust <- as.matrix(X[,clust]);
                 cors <- cor(X.clust,X[,p])
                 sqsums <- apply(X.clust,cols(),sqsum)
                 o <- order(cors,decreasing=TRUE);
                 data.frame(members = colnames(X)[clust][o],
                            correlation = cors[o],
                            sqsum = sqsums[o])})
  } else {
  mats <- lapply(detclust,function(clust) { X.clust <- as.matrix(X[,clust]);
                 cors <- cor(X.clust,X[,p])
                 sqsums <- apply(X.clust,cols(),sqsum)
                 o <- order(cors,decreasing=TRUE);
                 data.frame(members = clust[o],
                            correlation = cors[o],
                            sqsum = sqsums[o])})
  }
  
  det.tab <- mats[[1]]
  if(length(detclust)>1) {
    for (i in 2:length(detclust)) det.tab <- rbind(det.tab,mats[[i]])
  }
  details <- cbind(rep(names(detclust),unlist(lapply(detclust,length))),
                   det.tab)
  colnames(details) <- c("groups","members","correlation","sqsum")
  rownames(details) <- 1:nrow(details)

  ##############################################################################
  #                                                                            #
  # ordered.clust.names is computed differently if given gene sets are tested, #
  # i.e., 'grpdata' is not null because the 'idx' variable returned by         #
  # the C-program gives then the original order of the gene sets as they were  #
  # given in 'grpdata' and not the original order of the data matrix X         #
  #                                                                            #
  ##############################################################################

  if (is.null(grpdata)) {
      ordered.clust.names <- colnames(X)[res$idx[clustnum:1]] 
  } else {
      ordered.clust.names <- names(pg$grpdata)[res$idx[clustnum:1]] 
  }
 
  result <- data.frame(variable=ordered.clust.names, 
                       tval=res$testval[(clustnum):1],
                       pval=res$pval[(clustnum):1])
  if (file!="") {
    write.table(result, file=overview.file,quote=F,sep="\t")
    #if (grpdata.file!="")
      #write.table(SETGEN.grplist2tab(detclust),file=grpdata.file,quote=F,sep="\t")
    if (!is.null(details))
      write.table(details,file=details.file,quote=F,sep="\t")
  } 
  cvals <- res$critvals
  names(cvals)<-crit.vals
  invisible(return(
      list(summary=sum.list,
           overview=result,
           grpdata=detclust,
           details=mats,       
           crit.vals=cvals)))
}
, name = "SETGEN")
