/******************************************************************************/
/*                                                                            */
/* mathutils.[ch]                                                             */
/* ==============                                                             */
/*                                                                            */
/* Utility functions mathematical applicataions: Memory management for        */
/* vectors and matrices, compare functions, mathematical functions, random    */
/* number generation. Eigensystem calculation for symmetric matrices.         */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Standard header files.                                                     */
/*                                                                            */
/******************************************************************************/

#include <float.h>  /* For 'DBL_MIN' and 'DBL_MAX'.                           */
#include <limits.h> /* For 'LONG_MIN' and 'LONG_MAX'.                         */
#include <math.h>
#include <stdio.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>

/******************************************************************************/
/*                                                                            */
/* User header files.                                                         */
/*                                                                            */
/******************************************************************************/

#include "mathutils.h"

/******************************************************************************/
/*                                                                            */
/* Global variables.                                                          */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Seed variable for random number generation                                 */
/*                                                                            */
/******************************************************************************/

static int rseed=0L;

/******************************************************************************/
/*                                                                            */
/* Standard warning handler.                                                  */
/*                                                                            */
/******************************************************************************/

void mathwarning(char warning_text[]) {
  fprintf(stderr, "\nMath package run-time warning...");
  fprintf(stderr, "\n%s", warning_text);
  fprintf(stderr, "\n...now continuing...");
}

/******************************************************************************/
/*                                                                            */
/* Standard error handler. Exits to the system.                               */
/*                                                                            */
/******************************************************************************/

void matherror(char error_text[]) {
  fprintf(stderr, "\nMath package run-time error...");
  fprintf(stderr, "\n%s", error_text);
  fprintf(stderr, "\n...now exiting to system...");
  exit(1);
}

/******************************************************************************/
/*                                                                            */
/* The following functions serve for memory allocation of vectors of several  */
/* types. The first letter of the function indicates the type (c=char,        */
/* s=short, i=int, l=int, d=double) of the vector that is allocated. The     */
/* first index is 'nl' and the last index 'nh'.                               */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Allocate an unsigned Char VECTOR with subscript range v[nl..nh].           */
/*                                                                            */
/******************************************************************************/

unsigned char *cvector(int nl, int nh) {
  unsigned char *v;

  /****************************************************************************/
  /*                                                                          */
  /* Allocate memory range of length nh-nl+1.                                 */
  /*                                                                          */
  /****************************************************************************/

  v=(unsigned char *) malloc((size_t) ((nh-nl+1+M_END)*sizeof(unsigned char)));
  if (!v) matherror("allocation failure in cvector()");

  /****************************************************************************/
  /*                                                                          */
  /* If allocation succesful adjust index range from v[0..nh-nl] to           */
  /* v[nl..nh].                                                               */
  /*                                                                          */
  /****************************************************************************/

  return v-nl+M_END;
}

/******************************************************************************/
/*                                                                            */
/* Allocate a Short VECTOR with subscript range v[nl..nh].                    */
/*                                                                            */
/******************************************************************************/

short *svector(int nl, int nh) {
  short *v;

  /****************************************************************************/
  /*                                                                          */
  /* Allocate memory range of length nh-nl+1.                                 */
  /*                                                                          */
  /****************************************************************************/

  v=(short *) malloc((size_t) ((nh-nl+1+M_END)*sizeof(short)));
  if (!v) matherror("allocation failure in ivector()");

  /****************************************************************************/
  /*                                                                          */
  /* If allocation succesful adjust index range from v[0..nh-nl] to           */
  /* v[nl..nh].                                                               */
  /*                                                                          */
  /****************************************************************************/

  return v-nl+M_END;
}


/******************************************************************************/
/*                                                                            */
/* Allocate an Int VECTOR with subscript range v[nl..nh].                     */
/*                                                                            */
/******************************************************************************/

int *ivector(int nl, int nh) {
  int *v;

  /****************************************************************************/
  /*                                                                          */
  /* Allocate memory range of length nh-nl+1.                                 */
  /*                                                                          */
  /****************************************************************************/

  v=(int *) malloc((size_t) ((nh-nl+1+M_END)*sizeof(int)));
  if (!v) matherror("allocation failure in ivector()");

  /****************************************************************************/
  /*                                                                          */
  /* If allocation succesful adjust index range from v[0..nh-nl] to           */
  /* v[nl..nh].                                                               */
  /*                                                                          */
  /****************************************************************************/

  return v-nl+M_END;
}


/******************************************************************************/
/*                                                                            */
/* Allocate a Long VECTOR with subscript range v[nl..nh].                     */
/*                                                                            */
/******************************************************************************/

int *lvector(int nl, int nh) {
  int *v;

  /****************************************************************************/
  /*                                                                          */
  /* Allocate memory range of length nh-nl+1.                                 */
  /*                                                                          */
  /****************************************************************************/

  v=(int *) malloc((size_t) ((nh-nl+1+M_END)*sizeof(int)));
  if (!v) matherror("allocation failure in ivector()");

  /****************************************************************************/
  /*                                                                          */
  /* If allocation succesful adjust index range from v[0..nh-nl] to           */
  /* v[nl..nh].                                                               */
  /*                                                                          */
  /****************************************************************************/

  return v-nl+M_END;
}

/******************************************************************************/
/*                                                                            */
/* Allocate a Double VECTOR with subscript range v[nl..nh].                   */
/*                                                                            */
/******************************************************************************/

double *dvector(int nl, int nh) {
  double *v;

  /****************************************************************************/
  /*                                                                          */
  /* Allocate memory range of length nh-nl+1.                                 */
  /*                                                                          */
  /****************************************************************************/

  v=(double *) malloc((size_t) ((nh-nl+1+M_END)*sizeof(double)));
  if (!v) matherror("allocation failure in dvector()");

  /****************************************************************************/
  /*                                                                          */
  /* If allocation succesful adjust index range from v[0..nh-nl] to           */
  /* v[nl..nh].                                                               */
  /*                                                                          */
  /****************************************************************************/

  return v-nl+M_END;
}

/******************************************************************************/
/*                                                                            */
/* Allocate a short matrix with subscript range m[nrl..nrh][ncl..nch].        */
/*                                                                            */
/******************************************************************************/

short **smatrix(int nrl,int nrh,int ncl,int nch) {
  int i,nrow=nrh-nrl+1,ncol=nch-ncl+1;
  short **M;

  /****************************************************************************/
  /*                                                                          */
  /* Allocate 'nrow' pointers to rows.                                        */
  /*                                                                          */
  /****************************************************************************/

  M=(short **)malloc((size_t)((nrow+M_END)*sizeof(short *)));
  if (!M) matherror("allocation failure 1 in dmatrix()");
  M+=M_END;
  M-=nrl; /* Adjust row index.                                                */
  
  /****************************************************************************/
  /*                                                                          */
  /* Allocate data memory and set row pointers.                               */
  /*                                                                          */
  /****************************************************************************/

  M[nrl]=(short *)malloc((size_t)((nrow*ncol+M_END)*sizeof(short)));
  if (!M[nrl]) matherror("allocation failure 2 in dmatrix()");
  M[nrl]+=M_END;

  /****************************************************************************/
  /*                                                                          */
  /* Adjust column indices.                                                   */
  /*                                                                          */
  /****************************************************************************/

  M[nrl]-=ncl;
  for(i=nrl+1; i<=nrh; i++) M[i]=M[i-1]+ncol;

  /****************************************************************************/
  /*                                                                          */
  /* Return data matrix with index range [nrl..nrh][ncl..nch].                */
  /*                                                                          */
  /****************************************************************************/

  return M;
}


/******************************************************************************/
/*                                                                            */
/* Allocate an Int MATRIX with subscript range m[nrl..nrh][ncl..nch].         */
/*                                                                            */
/******************************************************************************/

int **imatrix(int nrl,int nrh,int ncl,int nch) {
  int i,nrow=nrh-nrl+1,ncol=nch-ncl+1;
  int **M;

  /****************************************************************************/
  /*                                                                          */
  /* Allocate 'nrow' pointers to rows.                                        */
  /*                                                                          */
  /****************************************************************************/

  M=(int **)malloc((size_t)((nrow+M_END)*sizeof(int *)));
  if (!M) matherror("allocation failure 1 in dmatrix()");
  M+=M_END;
  M-=nrl; /* Adjust row index.                                                */
  
  /****************************************************************************/
  /*                                                                          */
  /* Allocate data memory and set row pointers.                               */
  /*                                                                          */
  /****************************************************************************/

  M[nrl]=(int *)malloc((size_t)((nrow*ncol+M_END)*sizeof(int)));
  if (!M[nrl]) matherror("allocation failure 2 in dmatrix()");
  M[nrl]+=M_END;

  /****************************************************************************/
  /*                                                                          */
  /* Adjust column indices.                                                   */
  /*                                                                          */
  /****************************************************************************/

  M[nrl]-=ncl;
  for(i=nrl+1; i<=nrh; i++) M[i]=M[i-1]+ncol;

  /****************************************************************************/
  /*                                                                          */
  /* Return data matrix with index range [nrl..nrh][ncl..nch].                */
  /*                                                                          */
  /****************************************************************************/

  return M;
}


/******************************************************************************/
/*                                                                            */
/* Allocate a Double MATRIX with subscript range m[nrl..nrh][ncl..nch].       */
/*                                                                            */
/******************************************************************************/

double **dmatrix(int nrl,int nrh,int ncl,int nch) {
  int i,nrow=nrh-nrl+1,ncol=nch-ncl+1;
  double **M;

  /****************************************************************************/
  /*                                                                          */
  /* Allocate 'nrow' pointers to rows.                                        */
  /*                                                                          */
  /****************************************************************************/

  M=(double **) malloc((size_t) ((nrow+M_END)*sizeof(double *)));
  if (!M) matherror("allocation failure 1 in dmatrix()");
  M += M_END;
  M -= nrl; /* Adjust row index.                                              */
  
  /****************************************************************************/
  /*                                                                          */
  /* Allocate data memory and set row pointers.                               */
  /*                                                                          */
  /****************************************************************************/
  
  M[nrl]=(double *) malloc((size_t) ((nrow*ncol+M_END)*sizeof(double)));
  if (!M[nrl]) matherror("allocation failure 2 in dmatrix()");
  M[nrl] += M_END;

  /****************************************************************************/
  /*                                                                          */
  /* Adjust column indices.                                                   */
  /*                                                                          */
  /****************************************************************************/

  M[nrl] -= ncl;
  for(i=nrl+1; i <= nrh; i++) M[i]=M[i-1]+ncol;

  /****************************************************************************/
  /*                                                                          */
  /* Return data matrix with index range [nrl..nrh][ncl..nch].                */
  /*                                                                          */
  /****************************************************************************/

  return M;
}

/******************************************************************************/
/*                                                                            */
/* FREE an unsigned Char VECTOR allocated with 'cvector'.                     */
/*                                                                            */
/******************************************************************************/

void free_cvector(unsigned char *v,int nl) {

  /****************************************************************************/
  /*                                                                          */
  /* Free data range.                                                         */
  /*                                                                          */
  /****************************************************************************/

  free((FREE_ARG)(v+nl-M_END));
}

/******************************************************************************/
/*                                                                            */
/* FREE a Short VECTOR allocated with 'svector'.                              */
/*                                                                            */
/******************************************************************************/

void free_svector(short *v,int nl) {

  /****************************************************************************/
  /*                                                                          */
  /* Free data range.                                                         */
  /*                                                                          */
  /****************************************************************************/

  free((FREE_ARG)(v+nl-M_END));
}

/******************************************************************************/
/*                                                                            */
/* FREE an Int VECTOR allocated with 'ivector'.                               */
/*                                                                            */
/******************************************************************************/

void free_ivector(int *v,int nl) {

  /****************************************************************************/
  /*                                                                          */
  /* Free data range.                                                         */
  /*                                                                          */
  /****************************************************************************/

  free((FREE_ARG)(v+nl-M_END));
}

/******************************************************************************/
/*                                                                            */
/* FREE a Long VECTOR allocated with 'lvector'.                               */
/*                                                                            */
/******************************************************************************/

void free_lvector(int *v,int nl) {

  /****************************************************************************/
  /*                                                                          */
  /* Free data range.                                                         */
  /*                                                                          */
  /****************************************************************************/

  free((FREE_ARG)(v+nl-M_END));
}


/******************************************************************************/
/*                                                                            */
/* FREE a Double VECTOR allocated with 'dvector'.                             */
/*                                                                            */
/******************************************************************************/

void free_dvector(double *v,int nl) {

  /****************************************************************************/
  /*                                                                          */
  /* Free data range.                                                         */
  /*                                                                          */
  /****************************************************************************/

  free((FREE_ARG)(v+nl-M_END));
}

/******************************************************************************/
/*                                                                            */
/* FREE a Short MATRIX allocated with 'smatrix'.                              */
/*                                                                            */
/******************************************************************************/

void free_smatrix(short **M,int nrl,int ncl) {
  
  /****************************************************************************/
  /*                                                                          */
  /* Free data range.                                                         */
  /*                                                                          */
  /****************************************************************************/

  free((FREE_ARG)(M[nrl]+ncl-M_END));

  /****************************************************************************/
  /*                                                                          */
  /* Free pointers to rows.                                                   */
  /*                                                                          */
  /****************************************************************************/

  free((FREE_ARG)(M+nrl-M_END));
}  

/******************************************************************************/
/*                                                                            */
/* FREE an Int MATRIX allocated with 'imatrix'.                               */
/*                                                                            */
/******************************************************************************/

void free_imatrix(int **M,int nrl,int ncl) {

  /****************************************************************************/
  /*                                                                          */
  /* Free data range.                                                         */
  /*                                                                          */
  /****************************************************************************/

  free((FREE_ARG)(M[nrl]+ncl-M_END));

  /****************************************************************************/
  /*                                                                          */
  /* Free pointers to rows.                                                   */
  /*                                                                          */
  /****************************************************************************/

  free((FREE_ARG)(M+nrl-M_END));
}  

/******************************************************************************/
/*                                                                            */
/* FREE a Double MATRIX allocated with 'dmatrix'.                             */
/*                                                                            */
/******************************************************************************/

void free_dmatrix(double **M,int nrl,int ncl) {

  /****************************************************************************/
  /*                                                                          */
  /* Free data range.                                                         */
  /*                                                                          */
  /****************************************************************************/

  free((FREE_ARG)(M[nrl]+ncl-M_END));

  /****************************************************************************/
  /*                                                                          */
  /* Free pointers to rows.                                                   */
  /*                                                                          */
  /****************************************************************************/

  free((FREE_ARG)(M+nrl-M_END));
}  

/******************************************************************************/
/*                                                                            */
/* Merge two 'double' matrices by column (column numbers have to be equal).   */
/*                                                                            */
/******************************************************************************/

double *dcbind(double *X1, double *X2, int nrow, int ncol1, int ncol2) {
  double *X, *xi, *x1i=X1+(nrow-1)*ncol1+1, *x2i=X2+(nrow-1)*ncol2+1;

  X=dvector(1,nrow*(ncol1+ncol2));
  xi=X+(nrow-1)*(ncol1+ncol2)+ncol1+1;

  while(xi>X) {
    memcpy(xi,x2i,(size_t)(sizeof(double)*ncol2));
    xi-=ncol1;
    x2i-=ncol2;
    memcpy(xi,x1i,(size_t)(sizeof(double)*ncol1));
    xi-=ncol2;
    x1i-=ncol1;
  }

  return X;
}

/******************************************************************************/
/*                                                                            */
/* Functions for uniformly distributed random number generation. The code     */
/* stems from "Numerical Recipes in c". The double precision function calls   */
/* two times 'frunif'. In '?runiv' the random number is in the standard       */
/* interval (0,1). '...int' gives random numbers in the interval              */
/* ('min','max').                                                             */
/*                                                                            */
/******************************************************************************/

void initrseed(int rseedinit) {
  rseed=rseedinit;
}

int getrseed(){
  return rseed;
}

/******************************************************************************/
/*                                                                            */
/* Random value between 1 an 0 after "Numerical Recipes in C".                */
/*                                                                            */
/******************************************************************************/

#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

float frunif() {
  int j;
  int k;
  static int iy=0;
  static int iv[NTAB];
  float temp;
  
  if (rseed <= 0 || !iy) {
    if (-(rseed) < 1) rseed=1;
    else rseed = -(rseed);
    for (j=NTAB+7;j>=0;j--) {
      k=(rseed)/IQ;
      rseed=IA*(rseed-k*IQ)-IR*k;
      if (rseed < 0) rseed += IM;
      if (j < NTAB) iv[j] = rseed;
    }
    iy=iv[0];
  }
  k=(rseed)/IQ;
  rseed=IA*(rseed-k*IQ)-IR*k;
  if (rseed < 0) rseed += IM;
  j=iy/NDIV;
  iy=iv[j];
  iv[j] = rseed;
  if ((temp=AM*iy) > RNMX) return RNMX;
  else return temp;
}


/******************************************************************************/
/*                                                                            */
/* A more precise random number generator returning a 'double' value.         */
/*                                                                            */
/******************************************************************************/

double drunif() {
  return ((double)frunif()+(double)EPS*(double)frunif()+
	  (double)(EPS*EPS)*(double)frunif());
}

#undef IA
#undef IM
#undef AM
#undef IQ
#undef IR
#undef NTAB
#undef NDIV
#undef EPS
#undef RNMX

float  frunifint(float  l, float  h) {
  return 0;
}

double drunifint(double l, double h) {
  return 0;
}

/******************************************************************************/
/*                                                                            */
/* Function for (standard) normal distributed random number generation.       */
/*                                                                            */
/******************************************************************************/

double drnorm() {
  static int iset;
  static double gset;
  double fac,rsq,v1,v2;

  if (rseed < 0) iset=0;
  if (iset == 0) {
    do {
      v1=2.0*drunif()-1.0;
      v2=2.0*drunif()-1.0;
      rsq=v1*v1+v2*v2;
    } while (rsq >= 1.0 || rsq == 0.0);
    fac=sqrt(-2.0*log(rsq)/rsq);
    gset=v1*fac;
    iset=1;
    return v2*fac;
  } else {
    iset=0;
    return gset;
  }
}

double drnormmeansd(double mean, double sd) {
  return 0;
}

/******************************************************************************/
/*                                                                            */
/* Sampling function generates random sample of size 'sample_size' of an      */
/* index range from 'nl' to 'nh' with or without mulitples.                   */
/*                                                                            */
/******************************************************************************/

void lsample(int *sample, int nl, int nh, int samplesize, bool multiples) {
  int i, lrange=nh-nl+1, *tmpsample;
  double drange;
  double *unif;

  /****************************************************************************/
  /*                                                                          */
  /* Check if value range and desired sample size fit together.               */
  /*                                                                          */
  /****************************************************************************/

  if (!multiples && (lrange<samplesize))
    matherror("'samplesize' chosen to large");
  
  /****************************************************************************/
  /*                                                                          */
  /* If multiple copies of one value are acceptable random values are         */
  /* generated by multiplying the 'range' with a uniformly distributed random */
  /* number and adding the lower range limit.                                 */
  /*                                                                          */
  /****************************************************************************/

  if (multiples) {
    drange=(double)nh-(double)nl+1.;
    for (i=samplesize; i>0; i--) sample[i]=nl+(int)(drunif()*drange);
  } else {

    /**************************************************************************/
    /*                                                                        */
    /* If only unique values are allowed the 'sample'- or 'tmpsample'-vector  */
    /* is filled with the values from 'nl' to 'nh'. Then a vector of          */
    /* uniformly distributed random values of the same length like the        */
    /* 'sample'- or 'tmpsample'-vector is created. The two vectors are        */
    /* co-sorted using 'dlqsort2' so that the order of the 'sample'- or       */
    /* 'tmpsample'-values appears to be random.                               */
    /*                                                                        */
    /**************************************************************************/

    nl--;
    if (samplesize==lrange) {
      unif=dvector(1,samplesize);
      for (i=samplesize; i>0; i--) {
	unif[i]=drunif();
	sample[i]=nl+i;
      }
      dlqsort2(unif,sample,1,samplesize);
    } else {
      unif=dvector(1,lrange);
      tmpsample=lvector(1,lrange);
      for (i=lrange; i>0; i--) {
	unif[i]=drunif();
	tmpsample[i]=nl+i;
      }
      dlqsort2(unif,tmpsample,1,lrange);
      memcpy(sample+1,tmpsample+1,(size_t)(sizeof(int)*samplesize));
    }
  }
}

/******************************************************************************/
/*                                                                            */
/* Sampling function generates a sequence of 's' random sample of sizes from  */
/* 'sample_size' and index ranges from 'nl' to 'nh' with or without           */
/* mulitples.                                                                 */
/*                                                                            */
/******************************************************************************/

void lmultisample(int *multisample, int *nl, int *nh, int *samplesize, 
		  int s, bool multiples) {
  int i,offset=0;
  for (i=1; i<=s; i++) {
    lsample(multisample+offset,nl[i],nh[i],samplesize[i],multiples);
    offset+=samplesize[i];
  }
}

/******************************************************************************/
/*                                                                            */
/* The function lallsample generates all possible permutations of an index    */
/* range from nl to nh. Input is a sequence of nh-nl+1 elements initially     */
/* sorted such that: sample[nl]<=sample[nl+1]<=...<=sample[nh]. Output        */
/* is the next permutation in lexicocgraphic order. Output is stored in       */
/* the input vector sample. If input vector contains the last permutation     */
/* the vector 1,...,n must be given as the first input and the function must  */
/* be called n!-1 tumes. After each call the generated permutation is stored  */
/* in "sample".                                                               */
/* The algorithm first searches the input sequence from the end and finds     */
/* the index h of the first value such that sample[h] < sample[h+1].          */
/* Then it finds the largest index j such that sample[j] > sample[h].         */
/* The values sample[j] and sample[h] are switched and the order of           */
/* sample[h+1],..,sample[nh] is reversed.                                     */
/*                                                                            */
/******************************************************************************/

 void lallsample(int *sample, int nl, int nh) {
  int h, i, j, k, lrange=nh-nl+1, *tmpsample;
  bool b;

  h=lrange; 
  do {                                 // find h
      h-=1;
      b=(h>=1) ? sample[h]<sample[h+1] : 1;
  } while (!b);

  if (h>=1) {
      tmpsample=lvector(1,lrange);
      for (i=h; i<=lrange; i++) tmpsample[i]=sample[i];
      j=lrange;                       // find j and switch elem. j and h   
      while (tmpsample[j]<=tmpsample[h]) j-=1;
      sample[h]=tmpsample[j];
      tmpsample[j]=tmpsample[h];
      
      k=lrange;                       // reverse the order 
      for (i=h+1; i<=lrange; i++) {
           sample[i]=tmpsample[k];
           k-=1;
      }     
      free_lvector(tmpsample,1);
  } else mathwarning(" input is the last possible permutation "); 
}

/******************************************************************************/
/*                                                                            */
/* Rounding function. Rounds a 'int' number to its corresponding 'int'-     */
/* value.                                                                     */
/*                                                                            */
/******************************************************************************/

/* int lround(double x) {
  if ((x-floor(x)) < 0.5) return floor(x);
  else return ceil(x);
} */

/******************************************************************************/
/*                                                                            */
/* Rounding function. Rounds to nearest number with 'n' digits after decimal  */
/* point.                                                                     */
/*                                                                            */
/******************************************************************************/

double dround(double x, short n) {
  return (double)(floor(x*pow(10.,n)+.5))/pow(10.,n);}

/******************************************************************************/
/*                                                                            */
/* Find maximum value in a 'double' vector of length 'n'.                     */
/*                                                                            */
/******************************************************************************/

double dmax(double *x,int n) {

  /****************************************************************************/
  /*                                                                          */
  /* 'max' is initialised with the smallest 'double' value possible.          */
  /*                                                                          */
  /****************************************************************************/
  double *xi,max=-DBL_MAX;

  for (xi=&x[n]; xi>x; xi--) if (*xi > max) max=*xi;
  return max;
}

/******************************************************************************/
/*                                                                            */
/* Find minimum value in a 'double' vector of length 'n'.                     */
/*                                                                            */
/******************************************************************************/

double dmin(double *x, int n) {

  /****************************************************************************/
  /*                                                                          */
  /* 'min' is initialised with the largest 'double' value possible.           */
  /*                                                                          */
  /****************************************************************************/

  double *xi, min=DBL_MAX;

  for (xi=&x[n]; xi>x; xi--) if (*xi < min) min=*xi;
  return min;
}

/******************************************************************************/
/*                                                                            */
/* Find maximum value in a 'int' vector of length 'n'.                       */
/*                                                                            */
/******************************************************************************/

int lmax(int *x,int n) {

  /****************************************************************************/
  /*                                                                          */
  /* 'max' is initialised with the smallest 'double' value possible.          */
  /*                                                                          */
  /****************************************************************************/
  int *xi,max=INT_MIN;

  for (xi=&x[n]; xi>x; xi--) if (*xi > max) max=*xi;
  return max;
}

/******************************************************************************/
/*                                                                            */
/* Find minimum value in a 'int' vector of length 'n'.                       */
/*                                                                            */
/******************************************************************************/

int lmin(int *x, int n) {

  /****************************************************************************/
  /*                                                                          */
  /* 'min' is initialised with the largest 'int' value possible.             */
  /*                                                                          */
  /****************************************************************************/

  int *xi, min=INT_MAX;

  for (xi=&x[n]; xi>x; xi--) if (*xi < min) min=*xi;
  return min;
}


/******************************************************************************/
/*                                                                            */
/* Calculate mean value of a 'double' vector of length 'n'.                   */
/*                                                                            */
/******************************************************************************/

double dmean(double *x, int n) {
  double *xi, sum=0.;
  for (xi=&x[n]; xi>x; xi--) sum+=*xi;
  return sum/(double)n;
}

/******************************************************************************/
/*                                                                            */
/* Check whether a 'int' number 'e' is in a vector of 'int' numbers 'x' of  */
/* length 'n'.                                                                */
/*                                                                            */
/******************************************************************************/

bool is_lelem(int e,int *x, int n) {
  int *xi=&x[n];

  while (xi>x) if (e==*(xi--)) return TRUE;
  
  return FALSE;
}

/******************************************************************************/
/*                                                                            */
/* Reduction of a real symmetric matrix to tridiagonal form. The diagonals    */
/* above and below the main diagonal contain the same values. The reduction   */
/* is achieved using householder transformations. On output '**a' contains    */
/* transformation matrix (input matrix is destroyed), '*d' holds the main     */
/* diagonal values and '*e' contains the subdiagonal values.                  */
/*                                                                            */
/******************************************************************************/

void housered(double **a, int n, double *d, double *e) {
  int l,k,j,i;
  double scale,hh,h,g,f;

  for (i=n; i>=2; i--) {
    l=i-1;
    h=scale=0.0;
    if (l>1) {
      for (k=1; k<=l; k++) scale+=fabs(a[k][i]);
      if (scale==0.0) e[i]=a[l][i];
      else {
	for (k=1; k<=l; k++) {
	  a[k][i]/=scale;
	  h+=a[k][i]*a[k][i];
	}
	f=a[l][i];
	g=(f>=0.0 ? -sqrt(h) : sqrt(h));
	e[i]=scale*g;
	h-=f*g;
	a[l][i]=f-g;
	f=0.0;
	for (j=1; j<=l; j++) {
	  a[i][j]=a[j][i]/h;
	  g=0.0;
	  for (k=1; k<=j; k++) g+=a[k][j]*a[k][i];
	  for (k=j+1; k<=l; k++) g+=a[j][k]*a[k][i];
	  e[j]=g/h;
	  f+=e[j]*a[j][i];
	}
	hh=f/(h+h);
	for (j=1; j<=l; j++) {
	  f=a[j][i];
	  e[j]=g=e[j]-hh*f;
	  for (k=1; k<=j; k++) a[k][j]-=(f*e[k]+g*a[k][i]);
	}
      }
    } else e[i]=a[l][i];
    d[i]=h;
  }
  d[1]=0.0;
  e[1]=0.0;
  for (i=1; i<=n; i++) {
    l=i-1;
    if (d[i]) {
      for (j=1; j<=l; j++) {
	g=0.0;
	for (k=1; k<=l; k++) g+=a[k][i]*a[j][k];
	for (k=1; k<=l; k++) a[j][k]-=g*a[i][k];
      }
    }
    d[i]=a[i][i];
    a[i][i]=1.0;
    for (j=1; j<=l; j++) a[i][j]=a[j][i]=0.0;
  }
}

/******************************************************************************/
/*                                                                            */
/* Pythagoras function to calculate the length of the hypothenusis. The form  */
/* of calculation choosen here avoids multiplication overflow.                */
/*                                                                            */
/******************************************************************************/

double pythagoras(double a,double b) {
  double absa,absb;
  absa=fabs(a);
  absb=fabs(b);
  if (absa>absb) return absa*sqrt(1.0+SQR(absb/absa));
  else return (absb==0.0 ? 0.0 : absb*sqrt(1.0+SQR(absa/absb)));
}

/******************************************************************************/
/*                                                                            */
/* Solve eigensystem of a real tridiagonal matrix as calculated by 'housered' */
/* for real symmetric matrices. As input '*d' and '*e' contain the diagonal   */
/* respectively subdiagonal values. 'n' is the dimensionality of the matrix   */
/* and '**z' is the unit matrix or the transformation matrix produced by      */
/* 'housered' if the input matrix is not naturally tridiagonal. On output     */
/* '*d' holds the eigenvalues and '**z' holds the eigenvectors as row         */
/* vectors.                                                                   */
/*                                                                            */
/******************************************************************************/

void tridQLimpl(double d[],double e[],int n, double **z) {
  int m,l,iter,i,k;
  double s,r,p,g,f,dd,c,b;
  
  for (i=2; i<=n; i++) e[i-1]=e[i];
  e[n]=0.0;
  for (l=1; l<=n; l++) {
    iter=0;
    do {
      for (m=l; m<=n-1; m++) {
	dd=fabs(d[m])+fabs(d[m+1]);
	if ((fabs(e[m])+dd)==dd) break;
      }
      if (m!=l) {
	if (iter++ ==30) matherror("Too many iterations in 'tridQLimpl'");
	g=(d[l+1]-d[l])/(2.0*e[l]);
	r=pythagoras(g,1.0);
	g=d[m]-d[l]+e[l]/(g+SIGN(r,g));
	s=c=1.0;
	p=0.0;
	for (i=m-1; i>=l; i--) {
	  f=s*e[i];
	  b=c*e[i];
	  e[i+1]=(r=pythagoras(f,g));
	  if (r==0.0) {
	    d[i+1]-=p;
	    e[m]=0.0;
	    break;
	  }
	  s=f/r;
	  c=g/r;
	  g=d[i+1]-p;
	  r=(d[i]-g)*s+2.0*c*b;
	  d[i+1]=g+(p=s*r);
	  g=c*r-b;
	  for (k=1; k<=n; k++) {
	    f=z[i+1][k];
	    z[i+1][k]=s*z[i][k]+c*f;
	    z[i][k]=c*z[i][k]-s*f;
	  }
	}
	if (r==0.0 && i>=l) continue;
	d[l]-=p;
	e[l]=g;
	e[m]=0.0;
      }
    } while (m!=l);
  }
}

/******************************************************************************/
/*                                                                            */
/* Sorting function for eigensystems calculated with 'tridQLimpl'. The        */
/* eigenvalues and the according eigenvectors are sorted in decreasing order. */
/*                                                                            */
/******************************************************************************/

void eigsyssort(double d[],double **v, int n) {
  int k,j,i;
  double p;
  
  for (i=1; i<=n; i++) {
    p=d[k=i];
    for (j=i+1; j<=n; j++) if (d[j]>=p) p=d[k=j];
    if (k!=i) {
      d[k]=d[i];
      d[i]=p;
      for (j=1; j<=n; j++) {
	p=v[i][j];
	v[i][j]=v[k][j];
	v[k][j]=p;
      }
    }
  }
}

/******************************************************************************/
/*                                                                            */
/* Calculate eigensystem of a real, symmetric data matrix 'a' given as        */
/* concatenation of column vectors.                                           */
/* The result is stored in 'evecs' (eigen vectors) and evals (eigen values)   */
/* whereas the eigen vectors are stored as row vectors of length 'n'.         */
/*                                                                            */
/******************************************************************************/

void eigen(double *A, int n, double *evals, double *evecs) {
  int i,j;
  double **Amat, *d, *e;

  Amat=dmatrix(1,n,1,n);
  for (i=n; i>0; i--)
    for (j=n; j>0; j--) Amat[i][j]=A[(i-1)*n+j];

  d=dvector(1,n);
  e=dvector(1,n);

  /****************************************************************************/
  /*                                                                          */
  /* Householder reduction of 'Amat'.                                         */
  /*                                                                          */
  /****************************************************************************/

  housered(Amat,n,d,e);

  /****************************************************************************/
  /*                                                                          */
  /* Eigensystem solution.                                                    */
  /*                                                                          */
  /****************************************************************************/

  tridQLimpl(d,e,n,Amat);

  /****************************************************************************/
  /*                                                                          */
  /* Eigensystem sorting.                                                     */
  /*                                                                          */
  /****************************************************************************/

  eigsyssort(d,Amat,n);

  for (i=n; i>0; i--) {
    evals[i]=d[i];
    for (j=n; j>0; j--) evecs[(i-1)*n+j]=Amat[i][j];
  }

  free_dvector(d,1);
  free_dvector(e,1);
  free_dmatrix(Amat,1,1);

}

/******************************************************************************/
/*                                                                            */
/* Quicksort a vector of double values and with-sort a vector of 'int'       */
/* values. Function is used to generate randomly permutated index sets.       */
/*                                                                            */
/******************************************************************************/

void dlqsort2(double *mast, int *slav, int lo, int hi) {
  int l=lo, h=hi, p, ltmp;
  double pivot, dtmp;
  
  if (hi>lo) {
    p=lo+(int)((hi-lo+1)/2.);
    pivot=mast[p];           /* Get pivot element.                            */
    while (h>l) {
      while ((mast[l]<=pivot) && (l<p)) l++;
      while ((pivot<=mast[h]) && (p<h)) h--;
      if (l<h) {
	dtmp=mast[l];        /* Exchange elements of master list.             */
	mast[l]=mast[h];
	mast[h]=dtmp;
	ltmp=slav[l];        /* Exchange elements of slave list.              */
	slav[l]=slav[h];
	slav[h]=ltmp;
	if (l==p) p=h;       /* New pivot position.                           */
	else if (h==p) p=l;  /* New pivot position.                           */
      }      
    }
    
    if (lo<(p-1)) dlqsort2(mast,slav,lo,p-1);  /* Sort lower part of list.    */
    if (hi>(p+1)) dlqsort2(mast,slav,p+1,hi);  /* Sort upper part of list.    */
  }
}

/******************************************************************************/
/*                                                                            */
/* Quicksort a vector of 'int' values and with-sort a vector of 'int'       */
/* values. Function is used to keep track of original indices while sorting.  */
/*                                                                            */
/******************************************************************************/

void llqsort2(int *mast, int *slav, int lo, int hi) {
  int ltmpm,l=lo, h=hi, p,pivot, ltmps;
  
  if (hi>lo) {
    p=lo+(int)((hi-lo+1)/2.);
    pivot=mast[p];           /* Get pivot element.                            */
    while (h>l) {
      while ((mast[l]<=pivot) && (l<p)) l++;
      while ((pivot<=mast[h]) && (p<h)) h--;
      if (l<h) {
	ltmpm=mast[l];        /* Exchange elements of master list.            */
	mast[l]=mast[h];
	mast[h]=ltmpm;
	ltmps=slav[l];        /* Exchange elements of slave list.             */
	slav[l]=slav[h];
	slav[h]=ltmps;
	if (l==p) p=h;       /* New pivot position.                           */
	else if (h==p) p=l;  /* New pivot position.                           */
      }      
    }
    
    if (lo<(p-1)) llqsort2(mast,slav,lo,p-1);  /* Sort lower part of list.    */
    if (hi>(p+1)) llqsort2(mast,slav,p+1,hi);  /* Sort upper part of list.    */
  }
}

/******************************************************************************/
/*                                                                            */
/* Quicksort a vector of 'double' values.                                     */
/*                                                                            */
/******************************************************************************/

void dqsort(double *dvar, int lo, int hi) {
  int l=lo, h=hi, p;
  double pivot, tmp;
  
  if (hi>lo) {
    p=lo+(int)((hi-lo+1)/2.);
    pivot=dvar[p];           /* get pivot element                             */
    while (h>l) {
      while ((dvar[l]<=pivot) && (l<p)) l++;
      while ((pivot<=dvar[h]) && (p<h)) h--;
      if (l<h) {
	tmp=dvar[l];         /* exchange elements                             */
	dvar[l]=dvar[h];
	dvar[h]=tmp;
	if (l==p) p=h;       /* new pivot position                            */
	else if (h==p) p=l;  /* new pivot position                            */
      }      
    }
    
    if (lo<(p-1)) dqsort(dvar, lo, p-1);  /* sort lower part of list          */
    if (hi>(p+1)) dqsort(dvar, p+1, hi);  /* sort upper part of list          */
  }
}

/******************************************************************************/
/*                                                                            */
/* Select the kth smallest value in the array arr[1..n]. The input array will */
/* be rearranged to have this value in location arr[k], with smaller elements */
/* moved to arr[1..k-1] (in arbitrary order) and all larger elements in       */
/* arr[k+1..n] (also in arbitrary order)                                      */
/*                                                                            */
/******************************************************************************/

#define SWAP(a,b) temp=(a);(a)=(b);(b)=temp;

double select_kth_smallest(int k, int n, double arr[])
{
        int i,ir,j,l,mid;
        double a,temp;

        l=1;
        ir=n;
        for (;;) {
                if (ir <= l+1) {
                        if (ir == l+1 && arr[ir] < arr[l]) {
                                SWAP(arr[l],arr[ir])
                        }
                        return arr[k];
                } else {
                        mid=(l+ir) >> 1;
                        SWAP(arr[mid],arr[l+1])
                        if (arr[l] > arr[ir]) {
                                SWAP(arr[l],arr[ir])
                        }
                        if (arr[l+1] > arr[ir]) {
                                SWAP(arr[l+1],arr[ir])
                        }
                        if (arr[l] > arr[l+1]) {
                                SWAP(arr[l],arr[l+1])
                        }
                        i=l+1;
                        j=ir;
                        a=arr[l+1];
                        for (;;) {
                                do i++; while (arr[i] < a);
                                do j--; while (arr[j] > a);
                                if (j < i) break;
                                SWAP(arr[i],arr[j])
                        }
                        arr[l+1]=arr[j];
                        arr[j]=a;
                        if (j >= k) ir=j-1;
                        if (j <= k) l=i;
                }
        }
}
#undef SWAP

/******************************************************************************/
/*                                                                            */
/* Calculate the faculty of 'n'. The calculation is 'n' is limited to         */
/* 'MAXFAC'.                                                                  */
/*                                                                            */
/******************************************************************************/

int fac(int n) {
  int i, f=1;
  if (n<0) matherror("negative argument to 'fac'");
  else {
    if (n>MAXFAC) n=MAXFAC;
    for (i=2; i<=n; i++) f*=i;
  }
  return f;
}

/* Total correlation */
void cor_total(double *X,int n,int *ps,int g,double *COR) {
  int i,j,k;
  double *cori,sum,*xi,*xj,*xsum,*yi,*ysum;

  xsum=dvector(1,ps[1]);

  /* calculate square sums for X */
  for (i=1; i<=ps[1]; i++) {
    sum=0.;
    xi=X+(i-1)*n;
    for (k=1; k<=n; k++) sum+=xi[k]*xi[k];
    xsum[i]=sum;
  }

  /* one parameter group */
  if (g==1)
    for (i=1; i<=ps[1]; i++) {
      xi=X+(i-1)*n;
      COR[(i-1)*n+i]=1;
      for (j=i+1; j<=ps[1]; j++) {
	xj=X+(j-1)*n;
	sum=0.;
	for (k=1; k<=n; k++) sum+=xi[k]*xj[k];
	COR[(i-1)*n+j]=COR[(j-1)*n+i]=sum/sqrt(xsum[i]*xsum[j]);
      }
    }
  else if (g==2) {
    ysum=dvector(ps[1]+1,ps[1]+ps[2]);
    
    for (i=ps[1]+1; i<=ps[1]+ps[2]; i++) {
      sum=0.;
      xi=X+(i-1)*n;
      for (k=1; k<=n; k++) sum+=xi[k]*xi[k];
      ysum[i]=sum;
    }
   
    cori=COR+1;

    for (i=ps[1]+1; i<=ps[1]+ps[2]; i++) {
      sum=0.;
      yi=X+(i-1)*n;
      for (k=1; k<=n; k++) sum+=yi[k]*yi[k];
      ysum[i]=sum;
      for (j=1; j<=ps[1]; j++) {
	xj=X+(j-1)*n;
	sum=0;
	for (k=1; k<=n; k++) sum+=yi[k]*xj[k];
        *(cori++)=sum/sqrt(ysum[i]*xsum[j]);
      }
    }
    
    free_dvector(ysum,ps[1]+1);

  } else matherror("parameter group number not allowed");     

  free_dvector(xsum,1);

}  

int eg(double x,double y) {
  return (x>=y);
}

int le(double x,double y) {
  return (x<=y);
}
