/******************************************************************************/
/*                                                                            */
/* mathutils.[ch]                                                             */
/* ==============                                                             */
/*                                                                            */
/* Utility functions mathematical applicataions: Memory management for        */
/* vectors and matrices, compare functions, mathematical functions, random    */
/* number generation. Eigensystem calculation for symmetric matrices.         */
/*                                                                            */
/******************************************************************************/

#ifndef MATHUTILS_H
#define MATHUTILS_H

/******************************************************************************/
/*                                                                            */
/* Global constants and type definitions.                                     */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Boolean type emulation.                                                    */
/*                                                                            */
/******************************************************************************/

typedef enum {false=0,true=1} bool;

/******************************************************************************/
/*                                                                            */
/* Define constants for boolean type emulation ('TRUE' and 'FALSE').          */
/*                                                                            */
/******************************************************************************/

#define TRUE 1
#define FALSE 0

/******************************************************************************/
/*                                                                            */
/* Define constants for limitation values.                                    */
/*                                                                            */
/******************************************************************************/

#define MAXFAC 12

/******************************************************************************/
/*                                                                            */
/* Define constants used for memory allocation.                               */
/*                                                                            */
/******************************************************************************/

#define M_END 0
#define FREE_ARG char*

/******************************************************************************/
/*                                                                            */
/* Calculate the square of a value.                                           */
/*                                                                            */
/******************************************************************************/

static double sqrarg;
#define SQR(a) ((sqrarg=(a)) == 0.0 ? 0.0 : sqrarg*sqrarg)

/******************************************************************************/
/*                                                                            */
/* Set the sign of a value.                                                   */
/*                                                                            */
/******************************************************************************/

#define SIGN(a,b) ((b) >= 0.0 ? fabs(a) : -fabs(a))

/******************************************************************************/
/*                                                                            */
/* Compare two int values and return the maximum.                            */
/*                                                                            */
/******************************************************************************/

static int lmxa1, lmxa2; /* int max argument 1 & 2                          */
#define LMAX(a,b) (lmxa1=(a),lmxa2=(b),(lmxa1) > (lmxa2) ? (lmxa1) : (lmxa2))
 
/******************************************************************************/
/*                                                                            */
/* Compare two int values and return the minimum.                            */
/*                                                                            */
/******************************************************************************/

static int lmna1, lmna2; // int min argument 1 & 2
#define LMIN(a,b) (lmna1=(a),lmna2=(b),(lmna1) < (lmna2) ? (lmna1) : (lmna2))

/******************************************************************************/
/*                                                                            */
/* Compare two double values and return the minimum.                          */
/*                                                                            */
/******************************************************************************/

static double dmna1, dmna2; /* double min argument 1 & 2                      */
#define DMIN(a,b) (dmna1=(a),dmna2=(b),(dmna1) < (dmna2) ? (dmna1) : (dmna2))

/******************************************************************************/
/*                                                                            */
/* Standard warning handler.                                                  */
/*                                                                            */
/******************************************************************************/

void mathwarning(char error_text[]);

/******************************************************************************/
/*                                                                            */
/* Standard error handler. Exits to the system.                               */
/*                                                                            */
/******************************************************************************/

void matherror(char error_text[]);

/******************************************************************************/
/*                                                                            */
/* The following functions serve for memory allocation of vectors of several  */
/* types. The first letter of the function indicates the type (c=char,        */
/* s=short, i=int, l=int, d=double) of the vector that is allocated. The     */
/* first index is 'nl' and the last index 'nh'.                               */
/*                                                                            */
/******************************************************************************/

unsigned char *cvector(int nl,int nh);
short         *svector(int nl,int nh);
int           *ivector(int nl,int nh);
int          *lvector(int nl,int nh);
double        *dvector(int nl,int nh);

/******************************************************************************/
/*                                                                            */
/* The following functions serve for memory allocation of matrices of several */
/* types. The first letter of the function indicates the type (s=short,       */
/* l=int, d=double) of the matrix that is allocated. The first row index is  */
/* 'nrl' and the last row index 'nrh' and the first column index is 'ncl' and */
/* the last column index 'nch'.                                               */
/*                                                                            */
/******************************************************************************/

short  **smatrix(int nrl,int nrh,int ncl,int nch);
int    **imatrix(int nrl,int nrh,int ncl,int nch);
double **dmatrix(int nrl,int nrh,int ncl,int nch);

/******************************************************************************/
/*                                                                            */
/* Memory deallocation functions for vectors and matrices. The first and the  */
/* last (column/row) index has to be given to the corresponding function.     */
/*                                                                            */
/******************************************************************************/

void free_cvector(unsigned char *v,int nl);
void free_svector(short         *v,int nl);
void free_ivector(int           *v,int nl);
void free_lvector(int          *v,int nl);
void free_dvector(double        *v,int nl);
void free_smatrix(short        **M,int nrl,int ncl);
void free_imatrix(int          **M,int nrl,int ncl);
void free_dmatrix(double       **M,int nrl,int ncl);

/******************************************************************************/
/*                                                                            */
/* Merge two 'double' matrices by column (column numbers have to be equal).   */
/*                                                                            */
/******************************************************************************/

double *dcbind(double *X1, double *X2, int nrow, int ncol1, int ncol2); 

/******************************************************************************/
/*                                                                            */
/* Functions for uniformly distributed random number generation. The code     */
/* stems from "Numerical Recipes in c". The double precision function calls   */
/* two times 'frunif'. In '?runif' the random number is in the standard       */
/* interval (0,1). '...int' gives random numbers in the interval              */
/* ('min','max').                                                             */
/*                                                                            */
/******************************************************************************/

void initrseed(int rseedinit);
int getrseed();

float  frunif();
double drunif();

float  frunifint(float  l, float  h);
double drunifint(double l, double h);

/******************************************************************************/
/*                                                                            */
/* Function for (standard) normal distributed random number generation.       */
/*                                                                            */
/******************************************************************************/

double drnorm();

double drnormmeansd(double mean, double sd);

/******************************************************************************/
/*                                                                            */
/* Sampling function generates random sample of size 'sample_size' of an      */
/* index range from 'nl' to 'nh' with or without mulitples.                   */
/*                                                                            */
/******************************************************************************/

void lsample(int *sample, int nl, int nh, int sample_size, bool multiples);

/******************************************************************************/
/*                                                                            */
/* Sampling function generates a sequence of 's' random sample of sizes from  */
/* 'sample_size' and index ranges from 'nl' to 'nh' with or without           */
/* mulitples.                                                                 */
/*                                                                            */
/******************************************************************************/

void lmultisample(int *multisample, int *nl, int *nh, int *samplesize, 
		  int s, bool multiples);

/******************************************************************************/
/*                                                                            */
/* The function lallsample generates all possible permutations of an index    */
/* range from nl to nh. Input is a sequence of nh-nl+1 elements initially     */
/* sorted such that: sample[nl]<=sample[nl+1]<=...<=sample[nh], e.g. n1..nh. */
/*                                                                            */
/******************************************************************************/

void lallsample(int *sample, int nl, int nh);

/******************************************************************************/
/*                                                                            */
/* Rounding functions.                                                        */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Rounding function. Rounds a 'int' number to its corresponding 'int'-     */
/* value.                                                                     */
/*                                                                            */
/******************************************************************************/

/* int lround(double x); */

/******************************************************************************/
/*                                                                            */
/* Rounding function. Rounds to nearest number with 'n' digits after decimal  */
/* point.                                                                     */
/*                                                                            */
/******************************************************************************/

double dround(double x,short n);

/******************************************************************************/
/*                                                                            */
/* Vector characteristics functions. 'dmax' and 'dmin' return the maximum     */
/* respectively the minimum element of a 'double' vector of length 'n'.       */
/* 'dmean' gives the average of a 'double' vector of length 'n'.              */
/*                                                                            */
/******************************************************************************/

double dmax(double *x,int n);
double dmin(double *x,int n);

int lmax(int *x,int n);
int lmin(int *x,int n);

double dmean(double *x,int n);

/******************************************************************************/
/*                                                                            */
/* Check whether a 'int' number 'e' is in a vector of 'int' numbers 'x' of  */
/* length 'n'.                                                                */
/*                                                                            */
/******************************************************************************/

bool is_lelem(int e,int *x, int n);

/******************************************************************************/
/*                                                                            */
/* Reduction of a real symmetric matrix to tridiagonal form. The diagonals    */
/* above and below the main diagonal contain the same values. The reduction   */
/* is achieved using householder transformations. On output '**a' contains    */
/* transformation matrix (input matrix is destroyed), '*d' holds the main     */
/* diagonal values and '*e' contains the subdiagonal values.                  */
/*                                                                            */
/******************************************************************************/

void housered(double **a,int n,double *d,double *e);

/******************************************************************************/
/*                                                                            */
/* Pythagoras function to calculate the length of the hypothenusis. The form  */
/* of calculation choosen here avoids multiplication overflow.                */
/*                                                                            */
/******************************************************************************/

double pythagoras(double a,double b);

/******************************************************************************/
/*                                                                            */
/* Solve eigensystem of a real tridiagonal matrix as calculated by 'housered' */
/* for real symmetric matrices. As input '*d' and '*e' contain the diagonal   */
/* respectively subdiagonal values. 'n' is the dimensionality of the matrix   */
/* and '**z' is the unit matrix or the transformation matrix produced by      */
/* 'housered' if the input matrix is not naturally tridiagonal. On output     */
/* 'd[]' holds the eigenvalues and '**z' holds the eigenvectors as row        */
/* vectors.                                                                   */
/*                                                                            */
/******************************************************************************/

void tridQLimpl(double *d,double *e,int n,double **z);

/******************************************************************************/
/*                                                                            */
/* Sorting function for eigensystems calculated with 'tridQLimpl'. The        */
/* eigenvalues and the according eigenvectors are sorted in decreasing order. */
/*                                                                            */
/******************************************************************************/

void eigsyssort(double *d,double **v,int n);


/******************************************************************************/
/*                                                                            */
/* Calculate eigensystem of a real, symmetric data matrix 'a' given as        */
/* concatenation of column vectors.                                           */
/* The result is stored in 'evecs' (eigen vectors) and evals (eigen values)   */
/* whereas the eigen vectors are stored as row vectors of length 'n'.         */
/*                                                                            */
/******************************************************************************/

void eigen(double *A, int n, double *evals, double *evecs);

/******************************************************************************/
/*                                                                            */
/* The following various sorting routines sort their argument vectors in      */
/* increasing order.                                                          */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Quicksort a vector of 'double' values and with-sort a vector of 'int'     */
/* values. Function is used to generate randomly permutated index sets.       */
/*                                                                            */
/******************************************************************************/

void dlqsort2(double *mast, int *slav, int lo, int hi);

/******************************************************************************/
/*                                                                            */
/* Quicksort a vector of 'int' values and with-sort a vector of integer      */
/* values. Function is used to keep track of original indices while sorting.  */
/*                                                                            */
/******************************************************************************/

void llqsort2(int *mast, int *slav, int lo, int hi);

/******************************************************************************/
/*                                                                            */
/* Quicksort a vector of 'double' values.                                     */
/*                                                                            */
/******************************************************************************/

void dqsort(double *dvar, int lo, int hi);

/******************************************************************************/
/*                                                                            */
/* Select the kth smallest value in the array arr[1..n]. The input array will */
/* be rearranged to have this value in location arr[k], with smaller elements */
/* moved to arr[1..k-1] (in arbitrary order) and all larger elements in       */
/* arr[k+1..n] (also in arbitrary order)                                      */
/*                                                                            */
/******************************************************************************/

double select_kth_smallest(int k, int n, double arr[]);

/******************************************************************************/
/*                                                                            */
/* Calculate the faculty of 'n'. The calculation is 'n' is limited to         */
/* 'MAXFAC'.                                                                  */
/*                                                                            */
/******************************************************************************/

int fac(int n);

void cor_total(double *X,int n,int *ps,int g,double *COR);

int eg(double x,double y);
int le(double x,double y);

#endif
