/******************************************************************************/
/*                                                                            */
/* wycorrlaeuter.[ch]                                                         */
/* ==================                                                         */
/*                                                                            */
/* Implementation of the procedure by Juergen Laeuter for the search of gene  */
/* groups highly correlated to a given seed genes in a (multiclass)           */
/* microarray data set.                                                       */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Standard header files.                                                     */
/*                                                                            */
/******************************************************************************/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

/******************************************************************************/
/*                                                                            */
/* User header files.                                                         */
/*                                                                            */
/******************************************************************************/

#include "mathutils.h"
#include "wycorrlaeuter.h"

/******************************************************************************/
/*                                                                            */
/* Functions.                                                                 */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* A Westfall/Young procedure implementing the screening process described by */
/* Juergen Laeuter. In a first step all groups of correlated variables are    */
/* computed. For all those groups the sum of correlation coefficients of the  */
/* variables in a group with a seed variable are calculated. The sums are     */
/* taken as test values for the similarity of the groups and the seed         */
/* variable. The test values are subjected to a Westfall & Young resampling   */
/* procedure in order to correct for multiple testing.                        */
/*                                                                            */
/* This function uses PEARSON correlation for group building.                 */
/*                                                                            */
/* Function parameters:                                                       */
/*                                                                            */
/* X - a data matrix represented as vector of length 'n' x 'p' that is a      */
/*     concatenation of the 'p' column vectors of length 'n' of an original   */
/*     (n x p)-matrix. The seed variable is in the last column of 'X'.        */
/*                                                                            */
/* ns - the numbers of samples in the sample groups.                          */
/*                                                                            */
/* s - the number of sample groups.                                           */
/*                                                                            */
/* p - the number of variable.                                                */
/*                                                                            */
/* grpidx - if one wants to test not all the variable groups created around   */
/*          every variable (pivot) but only some given groups then grpidx is  */
/*          a vector of the variables in these groups.                        */
/*                                                                            */
/* glen - a vector of group lengths in 'grpidx'                               */
/*                                                                            */
/* gnum - number of given groups in 'grpidx'                                  */
/*                                                                            */
/* pivotidx - vector of indices of the pivot variables                        */
/*                                                                            */
/* rposonly - a boolean switch. If 'rposonly' is 'TRUE' only positive         */
/*            correlation between a pivot variable and the variables in its   */
/*            set is allowed. This condition is checked while creating the    */
/*            variable sets.                                                  */
/*                                                                            */
/* direction - if 1(-1) then the a one-sided test is performed, i.e., only    */
/*             clusters are taken into account which pivot variables are      */
/*             positively(negatively) correlated with the seed variable.      */
/*             Beta statistics of other clusters are set to a non-positive    */
/*             value. If 0 then the test is two-sided.                        */ 
/*                                                                            */
/* pivdom - a boolean switch. If 'pivdom' is 'TRUE' variance of the pivot     */
/*          variable is enforced. That suppresses symmetric group overhead.   */
/*                                                                            */ 
/* r2min - a threshold for the square of the correlation coefficient.         */
/*                                                                            */
/* nres - the number of resamplings that is determined inside the function.   */
/*                                                                            */
/* rand - a boolean variable deciding whether to seed the random number       */
/*        generator or not.                                                   */
/*                                                                            */
/* multisample - a boolean variable indicating whether the permutations       */
/*               should be restricted to the sample groups or not.            */
/*                                                                            */
/* rot - a boolean switch. If TRUE then the resampling is done with rotations */
/*       otherwise it is done with permutations                               */
/*                                                                            */
/* discret - a boolean swith. If TRUE then the truncated univariate statistics*/
/*           i.e. smaller than 'ucrit' or 'minbeta' will be set to 0 and the  */
/*           untruncated will be set to 1.                                    */
/*                                                                            */
/* trunc - a boolean switch. If TRUE then the the univariate statistics will  */
/*         be truncated.                                                      */
/*                                                                            */
/* ucrit - a critical threshold for a univariate B-statistics. Statistics     */
/*         smaller then that threshold will be truncated (set to 0)           */
/*                                                                            */
/* minbeta - another truncation threshold. Univariate B-statistics smaller    */
/*           then 'minbeta' will be truncated, i.e., set to 0.                */
/* samedirect - a boolean switch. If TRUE than the univariate B-statistics    */
/*              which do not correlate with the seed variable with the same   */
/*              sign as the pivot variable will be truncated.                 */
/* maxgene - number of greatest univariate statistics which will be used in   */
/*           the computation of the overall statistic of a gene set.          */
/*                                                                            */
/*                                                                            */
/*                                                                            */
/* clustidx - a vector containing the indices of variables clustered.         */
/*                                                                            */
/* clustlen - the numbers of variables in the clusters.                       */
/*                                                                            */
/* idx - As part of the W/Y procedure the variables are sorted by their test  */
/*       values. In order to keep track of the variables the new index of any */
/*       variable is stored at its old position in 'idx'.                     */
/*                                                                            */
/* testvalue - the values of the test function (sorted).                      */
/*                                                                            */
/* pvalue - the P-value of each test value obtained from the W/Y resampling   */
/*          distribution.                                                     */
/*                                                                            */
/* Return value(s):                                                           */
/*                                                                            */
/* All results are returned implicitely using the references of 'idx',        */
/* 'testvalue' and 'pvalue'.                                                  */
/*                                                                            */
/******************************************************************************/

void wy_corr_laeuter_pearson(double *X, int *ns, int s, int p, int *grpidx,
                             int *glen, int gnum, int *pivotidx,
                             bool rposonly,
			     int direction, bool pivdom, double r2min, 
			     int nres, bool rand, bool multisample, bool rot,
                             bool discret, bool trunc, double ucrit,
                             double minbeta, bool samedirect, int maxgene, 
                             bool singlestep,
			     int *idx, double *testvalue, double *pvalue,
                             double *critlevs, int ncrit, double *critvals) {
  int *clenoffset, clenoffsetidxi, clens, *clustidx, clustidxj, *clustlen, 
    clustnum, h, i, idxi, j, k, n=0, *nh, *nl, pcount, **permidx, *permidxh;
  double *dist, dn, dnres=(double)nres, r2, sum, sum2, test, x, *x2s, x2si, *xi, 
    *xj, *xp, *xmean, xmeanp, xmeani, *xs2, xs2p, xs2i, **xpresampled, 
    *xpresampledh;
  bool allperm=false; 
  FILE *outfile;

  /****************************************************************************/
  /*                                                                          */
  /* Init random number generator.                                            */
  /*                                                                          */
  /****************************************************************************/

  fprintf(stdout,"\n The C program starts rot =  %d \n", rot);
  fflush(stdout); 
 
  if (rand) initrseed(-time(0));
  else initrseed(0);

  /****************************************************************************/
  /*                                                                          */
  /* Calculate total sample number. Prepare multisample generation.           */
  /*                                                                          */
  /****************************************************************************/
  
  for (i=s; i>0; i--) n+=ns[i];
  dn=(double)n;
  if (multisample) {
    nl=lvector(1,s);
    nh=lvector(1,s);
    for (i=2, nl[1]=1, nh[1]=ns[1]; i<=s; i++) {
      nl[i]=nh[i-1]+1;
      nh[i]=nh[i-1]+ns[i];
    }
  }

  /****************************************************************************/
  /*                                                                          */
  /* Allocate memory for variable means and variances. Set pointers to last   */
  /* elements. Reduce 'p' by 1 in order to exclude the seed variable.         */
  /*                                                                          */
  /****************************************************************************/
  
  xmean=dvector(1,p);
  xs2=dvector(1,p);
  x2s=dvector(1,p);

  X+=n*p;
  xmean+=p;
  xs2+=p;
  x2s+=p;

  for (i=p; i>0; i--) {
    for (j=n, sum=0, sum2=0; j>0; j--) {
      x=*X;
      sum+=x;
      sum2+=x*x;
      X--;
    }
    *xmean=sum/dn;
    *(xs2--)=sum2-dn*(*xmean)*(*xmean);
    *(x2s--)=sum2;
    xmean--;
  }

  /****************************************************************************/
  /*                                                                          */
  /* If grpidx[1] == 0 , i.e., no variable groups to be tested have been      */
  /* given then all variables are used as pivot variables to create groups.   */
  /* Else, only the given groups are tested.                                  */
  /*                                                                          */  
  /****************************************************************************/
  
  if (grpidx[1] == 0) { 

    /**************************************************************************/
    /*                                                                        */
    /* Loop over all variables and calculate correlation between the pivot    */
    /* variable and all other variables. Count correlations above correlation */
    /* threshold and store index of correlated variables in 'clustidx'.       */
    /* 'clustnum' is the number of the groups                                 */
    /* (Note: Without the seed variable there are only 'p'-1 variables),      */
    /* 'clenoffset' is the vector of group offsets in 'clustidx', i.e.,       */
    /* 0,0+clustlen[1],0+clustlen[1]+clustlen[2],..,0+..+clustlen[clustnum-1] */
    /*                                                                        */
    /**************************************************************************/

    fprintf(stdout,"\n Creating correlated groups starts\n");
    fflush(stdout);

    clustnum=p-1;
    clustlen=lvector(1,clustnum);
    clenoffset=lvector(1,clustnum);
    clustidx=lvector(1,clustnum);

    for (i=1, clens=0; i<=clustnum; i++) {
      clenoffset[i]=clens;
      x2si=x2s[i];
      xmeani=xmean[i];
      xs2i=xs2[i];

      for (j=1; j<p; j++) {
      
        /**********************************************************************/
        /*                                                                    */
        /* Dynamically enlarge the allocated memory for clustidx. The         */
        /* enlargement is for 'p'-1 indices. So there may be a memory overhang*/
        /* that has to be corrected at the end of cluster generation.         */
        /*                                                                    */
        /**********************************************************************/
 
        if (!(clens%(p-1))) {
          clustidx++;
          clustidx=(int*)realloc(clustidx,(size_t)(sizeof(int)*(clens+p-1)));
          clustidx--;
        }

        // Perhaps it is faster to compute the correlation corr(i,i) p times 
        // than to check p*p times for i==j, but the check is done because
        // then it is guaranteed that the pivot variables are in the created 
        // sets. Otherwise, if r2min was for numerical reasons slightly greater 
        // than 1 the pivot variables could be left out.  
        if (i==j) {
	  clustidx[++clens]=j;
	  continue;
        }

        /**********************************************************************/
        /*                                                                    */
        /* If variance dominance of the pivot variable is requested. The      */
        /* variances of all the other variables are inspected first. If the   */
        /* pivot variable has a variance less than the other variable the     */
        /* current loop is skipped.                                           */
        /*                                                                    */
        /**********************************************************************/

        if (pivdom) if (x2si<x2s[j]) continue;
      
        xi=X+i*n;
        xj=X+j*n;
      
        /**********************************************************************/
        /*                                                                    */
        /* Sum up xi*yi.                                                      */
        /*                                                                    */
        /**********************************************************************/
      
        for (k=n, sum=0.; k>0; k--) sum+=*(xi--)*(*(xj--));
      
        /**********************************************************************/
        /*                                                                    */
        /* Calculate the SQUARED PEARSON correlation coefficient.             */
        /* The coefficient is computed in two stages:                         */
        /*   First the numerator of the unsquared correlation coefficient     */
        /* is computed. The sign of the numerator determines the sign of the  */
        /* correlation coefficient.                                           */
        /* If only positive correlations should be taken into account 'r' is  */
        /* compared with zero. If 'rposonly' equals 'TRUE' but the correlation*/
        /* coefficient is negative the current loop repetition is skipped.    */
        /*   Second, the numerator is multiplied with itself and divided by   */
        /* squared denominator of the correlation coefficient                 */  
        /*                                                                    */
        /**********************************************************************/
      
        //r2=(sum-dn*xmeani*xmean[j]);
        r2=sum;
        if (rposonly) if (r2<0) continue;
        //r2*=r2/(xs2i*xs2[j]);
        r2*=r2/(x2s[i]*x2s[j]);
      
        if (r2>=r2min) clustidx[++clens]=j;
      }
      clustlen[i]=clens-clenoffset[i];
    }

    /**************************************************************************/
    /*                                                                        */
    /* Fit cluster variable memory allocation.                                */
    /*                                                                        */
    /**************************************************************************/

    fprintf(stdout, "\n Fitting cluster variable memory allocation starts \n");
    fflush(stdout);


    clustidx++;
    clustidx=(int*)realloc(clustidx,(size_t)(sizeof(int)*clens));
    clustidx--;
  
    /*************************************************************************/
    /* Else, if grpidx != 0, i.e., some variable groups have been given      */
    /* then take the parameters of the groups.                               */
    /* If rposonly or r2min are specified then trim the variable sets to     */
    /* those genes which correlate with their pivot genes according to these */
    /* parameters. The new variable sets are output directly to clustidx,    */
    /* clustlen, clenoffset. The new length of the clustidx vector           */
    /* is returned as clens. After trimming the memory occupied by clustidx  */
    /* in memory is adjusted.                                                */ 
    /*                                                                       */
    /*************************************************************************/

  } else {     
      clenoffset = lvector(1,gnum);
      //if (rposonly || r2min>0.00001) {
      //    clens = trimsets(X, n, p, x2s, gnum, pivotidx, r2min, rposonly,
      //                     grpidx, glen, clustidx, clustlen, clenoffset);
      //clustidx++;
      //clustidx=(int*)realloc(clustidx,(size_t)(sizeof(int)*clens));
      //clustidx--;
      //} else {      
          clustnum = gnum;
          clustlen = glen;   
          clustidx = grpidx;
          for (i=1, clens=0; i<=gnum; i++) {
               clenoffset[i] = clens;
               clens += clustlen[i];
          }
      //}
  }

  /****************************************************************************/
  /*                                                                          */
  /* Calculate the test values for all groups, that is the sum of the squared */
  /* correlation coefficients of the variables of a cluster with the seed     */
  /* variable.                                                                */
  /* In the case of a one-sided test (direction != 0) the squared correlation */
  /* of the pivot variable with the target gene is artificially set to -p     */
  /* which guarantees that the beta statistic is not positive and, therefore, */
  /* the cluster of that pivot variable receives a p-value euqal to 1         */ 
  /* 'compute_betasum_maxbeta' is adapted to the case when the number of      */
  /* variables of a set contributing to its statistic is constrained.         */
  /* if gene sets to be tested are not given, i.e. grpidx[1]==0, then         */
  /* the index of the pivot variable is 'i'.Else it is pivotidx[i].           */ 
  /*                                                                          */
  /****************************************************************************/

  fprintf(stdout, "\n clens= %ld Calculating test values starts \n", clens);
  fflush(stdout);  

  xmeanp=xmean[p];
  xs2p=xs2[p]; 

  if (grpidx[1] == 0) { 
      if (maxgene == 0) {
          for (i=clustnum; i>0; i--) {
              testvalue[i] = compute_betasum(X, n, p, X+n*(p-1), 
                                         clustidx, clustlen[i], 
                                         clenoffset[i], x2s, i, direction, 
                                         discret, trunc, ucrit, minbeta,
                                         samedirect);
          }
      } else {
          for (i=clustnum; i>0; i--) {
              testvalue[i] = compute_betasum_from_maxbeta(X, n, p, X+n*(p-1),
                                         clustidx, clustlen[i],
                                         clenoffset[i], x2s, i, direction,
                                         discret, trunc, ucrit, minbeta,
                                         samedirect, maxgene);
          }
      }
  } else {
      if (maxgene == 0) {
          for (i=clustnum; i>0; i--) {
              testvalue[i] = compute_betasum(X, n, p, X+n*(p-1), 
                                         clustidx, clustlen[i], 
                                         clenoffset[i], x2s, pivotidx[i], direction, 
                                         discret, trunc, ucrit, minbeta,
                                         samedirect);
          }
      } else {
          for (i=clustnum; i>0; i--) {
              testvalue[i] = compute_betasum_from_maxbeta(X, n, p, X+n*(p-1),
                                         clustidx, clustlen[i],
                                         clenoffset[i], x2s, pivotidx[i], direction,
                                         discret, trunc, ucrit, minbeta,
                                         samedirect, maxgene);
          }
      }
  }

  /****************************************************************************/
  /*                                                                          */
  /* This is a different way of implementing the one-sided test. It may be    */
  /* a little less efficient if the sample size is large.                     */
  /* if direction!=0 then a one-sided test is performed. If direction==1(-1)  */
  /* then the correlation of every pivot variable with the target gene must   */
  /* be positive(negative) and the test statistics of clusters where it is    */ 
  /* not the case are set to a value not greater than 0 (e.g. here -1)        */ 
  /*                                                                          */ 
  /****************************************************************************/

  //if (direction != 0) { 
  //    for (i=p-1; i>0; i--) {
  //         xp=X+n*p;
  //         xj=X+n*i;
  //         for (k=n, sum=0.; k>0; k--) sum+=*(xp--)*(*(xj--));
  //         if (direction*sum<0) 
  //             testvalue[i]=-1.0;
  //    }
  //}

  /****************************************************************************/
  /*                                                                          */
  /* Sort the testvalues in increasing order. With-sort the index vector to   */
  /* keep track of the old variable position.                                 */
  /* If 'grpdata' is given then 'idx' is the vector of indices of the original*/
  /* order of the given groups. Else, the indices show the original order     */
  /* of variables in the data matrix, because a group is constructed around   */
  /* every variable.                                                          */
  /*                                                                          */
  /****************************************************************************/
  
  for (i=1; i<=clustnum; i++)
       idx[i] = i;

  fprintf(stdout,"\n Sorting starts\n");
  fflush(stdout); 

  dlqsort2(testvalue,idx,1,clustnum);

  /****************************************************************************/
  /*                                                                          */ 
  /* if nres==0 then all possible permutations should be generated.           */
  /*                                                                          */
  /****************************************************************************/

  if (nres==0) {
      if (n>MAXFAC) {
          mathwarning("too many samples (n=12) for generating \
                       all permutations. Continuing with 1000 ramdom\
                       permutations.");
          nres=1000;
          dnres=1000.0;
      } else {
          nres=fac(n);
          dnres=(double)nres;
          allperm=true;
      } 
  }

  /****************************************************************************/
  /*                                                                          */
  /* Allocate a vector of pointers to resampled versions of the seed variable.*/
  /* The resampled versions can be created by either permuting or rotating    */
  /* the original seed variable vector                                        */  
  /*                                                                          */     
  /****************************************************************************/

  xpresampled=(double **)malloc((size_t)(sizeof(double *)*(nres-1)));
  xpresampled-=2;

  /****************************************************************************/
  /*                                                                          */
  /* initialize the vector of resampled seed variable with rotation vectors   */
  /* the seed variable is not needed because if it is fixed then the rotation */
  /* vectors are standard spherically distributed. Rotation vectors are       */
  /* multiplied here by square root of the sum of squares of the seed variable*/
  /* x2s[p] but while computing test statistics the rotations vectors are     */
  /* divided by the same number. The reason for this multiplying and then     */
  /* dividing is that in this way the same code which computes                */
  /* the test statistic from permuted data can be used also on rotated data   */
  /*                                                                          */
  /****************************************************************************/

  if (rot) {
      
      for (h=2; h<=nres; h++) {
           xpresampled[h]=dvector(1,n);
           xpresampledh=xpresampled[h];
           for (k=1, sum=0.; k<=n; k++) {
                *(++xpresampledh)=x=drnorm(); 
                sum+=x*x;
           }
           sum=sqrt(x2s[p]/sum);
           for (k=n; k>0; k--) (*(xpresampledh--))*=sum;
      }    
  } else {

    /**************************************************************************/
    /*                                                                        */
    /* Else initialize the vector of resampled seed variable with permuted    */
    /* seed variable vectors.                                                 */ 
    /*                                                                        */
    /**************************************************************************/
  
      fprintf(stdout,"\n Permuting starts \n");
      fflush(stdout);


    /**************************************************************************/
    /*                                                                        */
    /* The data structure to yield the permutation index vectors is a vector  */
    /* of pointers to vectors. Each vector contains a permutation of the n    */
    /* indices 1...n.                                                         */
    /* The type of such a construct is "int **" (pointer to a vector of      */
    /* pointers to "int". As each element of "permidx" is a pointer to a     */
    /* vector of "int" values they have to be allocated separately.          */
    /*                                                                        */
    /**************************************************************************/

      permidx=(int **)malloc((size_t)(sizeof(int *)*(nres-1)));
      permidx-=2;
  
      if (allperm) {   // all permutations

    /**************************************************************************/
    /* init the permutation index vector with 1,..,n                          */
    /**************************************************************************/
 
          int* tmpidx=lvector(1,n);              
          for (i=1; i<=n; i++) tmpidx[i]=i;   

    /**************************************************************************/
    /* generate next permutation of the index vector and copy it to permidx[h]*/
    /**************************************************************************/
     
          for (h=2; h<=nres; h++) {
               permidx[h]=lvector(1,n);
               lallsample(tmpidx,1,n);
               for (i=1; i<=n; i++) permidx[h][i]=tmpidx[i];
          }
          free_lvector(tmpidx,1);
      } else         // random permutations
          for (h=2; h<=nres; h++) {
            permidx[h]=lvector(1,n);
            if (multisample) lmultisample(permidx[h],nl,nh,ns,s,FALSE); 
            else lsample(permidx[h],1,n,n,FALSE);
          }

    /**************************************************************************/
    /*                                                                        */               
    /* initialize resampled versions of the seed variable with permutations   */ 
    /*                                                                        */
    /**************************************************************************/

      xp=X+n*(p-1);
      for (h=2; h<=nres; h++) {
           xpresampled[h]=dvector(1,n);
           permidxh=permidx[h];
           for (k=1; k<=n; k++) 
                xpresampled[h][k]=xp[permidxh[k]];
      }

    /**************************************************************************/
    /*                                                                        */ 
    /* Free memory allocated for the permutation index vectors.               */
    /* End of the resampled seed variable initialization.                     */
    /* (with permutations or rotations depending on the value of rot)         */
    /*                                                                        */
    /**************************************************************************/

      for (h=2; h<=nres;h++) free_lvector(permidx[h],1);
      free(permidx+2);
  }          

  /****************************************************************************/
  /*                                                                          */
  /* Allocate and initialize distribution vector                              */
  /*                                                                          */
  /****************************************************************************/

  dist=dvector(1,nres);
  for (h=nres; h>0; h--) dist[h]=0.;

  /****************************************************************************/
  /*                                                                          */
  /* The following lines do the W/Y resampling. The process starts with the   */
  /* "worst" variable. For that variable all resampling variants are taken    */
  /* and for each variant the test value is calculated. The test values are   */
  /* stored in the 'dist'-variable. Finally, the P-value of the variable is   */
  /* calculated using the distribution stored in 'dist'.                      */
  /* Then the next best variable is taken and it goes through the same process*/
  /* (the resampling order has to be the same) and for each resampling data   */
  /* the 'dist'-variable is updated if necessary (that means a larger test    */
  /* value has been found).                                                   */
  /*                                                                          */
  /****************************************************************************/
  
  fprintf(stdout,"\n");
  fflush(stdout);
  xp=X+n*(p-1);

  for (i=1; i<=clustnum; i++) {

    /**************************************************************************/
    /*                                                                        */
    /* Get gene index of the original (unsorted) order get appropriate sum of */
    /* the values of that gene.                                               */
    /*                                                                        */
    /**************************************************************************/

    idxi=idx[i];
    clenoffsetidxi=clenoffset[idxi];
    fprintf(stdout,"\rProgress: %6.2lf %%",(double)i/(double)clustnum*100.);
    fflush(stdout);

    /**************************************************************************/
    /*                                                                        */
    /* Set 'pcount' to one. That is possible because the original test values */
    /* beint to the resampling distribution. So there is at least one value  */
    /* greater than or equal to the test value.                               */
    /* If the test value of the current gene is better than the appropriate   */
    /* value of the distribution the distribution variable 'dist' is updated. */
    /*                                                                        */
    /**************************************************************************/

    pcount=1;
    if (dist[1]<testvalue[i]) dist[1]=testvalue[i];

    /**************************************************************************/
    /*                                                                        */
    /* Loop through all resamplings.                                          */
    /*                                                                        */
    /**************************************************************************/
    
    for (h=2; h<=nres; h++) {
      
      /************************************************************************/
      /*                                                                      */
      /* Get permutation index for the current resampling.                    */
      /* As above, direction != 0 results in a one-sided test.                */
      /* 'compute_betasum_maxbeta' is adapted to the case when the number of  */
      /* variables of a set contributing to its statistic is constrained.     */
      /* if gene sets to be tested are not given, i.e. grpidx[1]==0, then     */
      /* the index of the pivot variable is 'idxi'.Else it is pivotidx[idxi]. */ 
      /*                                                                      */
      /************************************************************************/
      
      
      if (grpidx[1] == 0) { 
          if (maxgene == 0) {
              test = compute_betasum(X, n, p, xpresampled[h], 
                                 clustidx, clustlen[idxi],
                                 clenoffset[idxi], x2s, idxi, direction,
                                 discret, trunc, ucrit, minbeta, samedirect);
          } else {
              test = compute_betasum_from_maxbeta(X, n, p, xpresampled[h],
                                 clustidx, clustlen[idxi],
                                 clenoffset[idxi], x2s, idxi, direction,
                                 discret, trunc, ucrit, minbeta,
                                 samedirect, maxgene);    
          }
      } else {
          if (maxgene == 0) {
              test = compute_betasum(X, n, p, xpresampled[h], 
                                 clustidx, clustlen[idxi],
                                 clenoffset[idxi], x2s, pivotidx[idxi], 
                                 direction,
                                 discret, trunc, ucrit, minbeta, samedirect);
          } else {
              test = compute_betasum_from_maxbeta(X, n, p, xpresampled[h],
                                 clustidx, clustlen[idxi],
                                 clenoffset[idxi], x2s, pivotidx[idxi], 
                                 direction,
                                 discret, trunc, ucrit, minbeta,
                                 samedirect, maxgene);    
          }
      }

      /************************************************************************/
      /*                                                                      */
      /* if direction!=0 then a one-sided test is performed.                  */
      /* If direction==1(-1) then the correlation of every pivot variable     */
      /* with the target gene must be positive(negative) and the test         */
      /* statistics of clusters where it is not the case are set to a value   */
      /* not greater than 0 (e.g. here -1)                                    */ 
      /*                                                                      */ 
      /************************************************************************/

       //if (direction != 0) { 
       //         xj=X+n*(idxi-1);
       //         for (k=n, sum=0.; k>0; k--) sum+=xpresampledh[k]*xj[k];
       //         if (direction*sum<0) 
       //             test=-1.0;
       //}

      /************************************************************************/
      /*                                                                      */
      /* If the test value is greater than the maximum of the test values of  */
      /* former variables the distribution is updated.                        */
      /*                                                                      */
      /************************************************************************/

      if (dist[h]<test) dist[h]=test;
      
      /************************************************************************/
      /*                                                                      */
      /* If the original test value of the current variable is less than or   */
      /* equal to the distribution value of the current resampling 'pcount'   */
      /* is incremented. (With 'pcount' the empiric P-value is calculated.)   */
      /*                                                                      */
      /************************************************************************/

      if (testvalue[i]<=dist[h]) pcount++;
    }

    pvalue[i]=(double)pcount/dnres;
  }

  /****************************************************************************/
  /* This loop is a makeshift solution to compute one-step and not step-down  */
  /* p-values !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!                             */
  /****************************************************************************/

  if(singlestep) {
      for(i=1;i<=clustnum;i++) {
          pcount=1;
          for(h=2;h<=nres;h++) {
              if (testvalue[i]<=dist[h]) pcount++;
          }
        pvalue[i]=(double)pcount/dnres;
      } 
  }

  /****************************************************************************/

  double mindist=1000.0, maxdist=0.0;
  int hmin=0,hmax=0;

  for (h=nres; h>0; h--) {
       if (mindist > dist[h]) {  
           mindist=dist[h];
           hmin=h;
       }
       if (maxdist < dist[h]) {
            maxdist=dist[h];
            hmax=h; 
       }
  }

  fprintf(stdout, 
  "\n Min of the permutation distribution of the maximal statistics: %lf, at permutation: %d", 
      mindist, hmin);
  fprintf(stdout, 
      "\n Max of the permutation distribution of the maximal statistics: %lf, at permutation: %d", 
      maxdist, hmax);
  fflush(stdout);

  /****************************************************************************/
  /*                                                                          */
  /* Adjust P-values for monotony so that P[i+1] <= P[i].                     */
  /*                                                                          */
  /****************************************************************************/

  for (i=clustnum-1; i>0; i--)
    if (pvalue[i+1] > pvalue[i]) pvalue[i]=pvalue[i+1];

  /****************************************************************************/
  /*                                                                          */
  /* Calculate critical values.                                               */
  /*                                                                          */
  /****************************************************************************/

  dqsort(dist,1,nres);

  for (i=1; i<=ncrit; i++) {
    critvals[i]=dist[(int)ceil(dnres*(1.-critlevs[i]))];
  }

  /****************************************************************************/
  /*                                                                          */
  /* Free locally allocated memory.                                           */
  /*                                                                          */
  /****************************************************************************/
  
  free_dvector(dist,1);
  free_dvector(xmean,1);
  free_dvector(xs2,1);
  free_dvector(x2s,1);

  for (h=2; h<=nres;h++) free_dvector(xpresampled[h],1);
  free(xpresampled+2);
  if (multisample) {
    free_lvector(nl,1);
    free_lvector(nh,1);
  }
  free_lvector(clenoffset,1);

  if (grpidx[1] == 0) {
      free_lvector(clustidx,1);
      free_lvector(clustlen,1);
  } else {
      clustidx = NULL;
      clustlen = NULL;
  }

  fprintf(stdout,"\n");
  fflush(stdout);
}

/******************************************************************************/
/*                                                                            */
/* Functions.                                                                 */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* A Westfall/Young procedure implementing the screening process described by */
/* Juergen Laeuter. In a first step all groups of correlated variables are    */
/* computed. For all those groups the sum of correlation coefficients of the  */
/* variables in a group with a seed variable are calculated. The sums are     */
/* taken as test values for the similarity of the groups and the seed         */
/* variable. The test values are subjected to a Westfall & Young resampling   */
/* procedure in order to correct for multiple testing.                        */
/*                                                                            */
/* This function uses total correlation for group building.                   */
/*                                                                            */
/* Function parameters:                                                       */
/*                                                                            */
/* X - a data matrix represented as vector of length 'n' x 'p' that is a      */
/*     concatenation of the 'p' column vectors of length 'n' of an original   */
/*     (n x p)-matrix. The seed variable is in the last column of 'X'.        */
/*                                                                            */
/* ns - the numbers of samples in the sample groups.                          */
/*                                                                            */
/* s - the number of sample groups.                                           */
/*                                                                            */
/* p - the number of variable.                                                */
/*                                                                            */
/* nres - the number of resamplings that is determined inside the function.   */
/*                                                                            */
/* rand - a boolean variable deciding whether to seed the random number       */
/*        generator or not.                                                   */
/*                                                                            */
/* multisample - a boolean variable indicating whether the permutations       */
/*               should be restricted to the sample groups or not.            */
/*                                                                            */
/* rposonly - a boolean switch. If 'rposonly' is 'TRUE' only positive         */
/*            correlations are taken into account.                            */
/*                                                                            */
/* pivdom - a boolean switch. If 'pivdom' is 'TRUE' variance of the pivot     */
/*          variable is enforced. That suppresses symmetric group overhead.   */
/*                                                                            */
/* r2min - a threshold for the square of the correlation coefficient.         */
/*                                                                            */
/* clustidx - a vector containing the indices of variables clustered.         */
/*                                                                            */
/* clustlen - the numbers of variables in the clusters.                       */
/*                                                                            */
/* idx - As part of the W/Y procedure the variables are sorted by their test  */
/*       values. In order to keep track of the variables the new index of any */
/*       variable is stored at its old position in 'idx'.                     */
/*                                                                            */
/* testvalue - the values of the test function (sorted).                      */
/*                                                                            */
/* pvalue - the P-value of each test value obtained from the W/Y resampling   */
/*          distribution.                                                     */
/*                                                                            */
/* Return value(s):                                                           */
/*                                                                            */
/* All results are returned implicitely using the references of 'idx',        */
/* 'testvalue' and 'pvalue'.                                                  */
/*                                                                            */
/******************************************************************************/

void wy_corr_laeuter_total(double *X, int *ns, int s, int p, bool rposonly,
			   bool pivdom, double r2min, 
			   int nres, bool rand, bool multisample, 
			   int *idx, double *testvalue, double *pvalue,
                           double *critlevs, int ncrit, double *critvals) {
  int *clenoffset, clenoffsetidxi, clens, *clustidx, clustidxj, *clustlen, h, 
    i, idxi, j, k, n=0, *nh, *nl, pcount, **permidx, *permidxh;
  double *dist, dn, dnres=(double)nres, r, sum, sum2, test, x, *x2s, *xi, *xj,
    *xp, *xmean, xmeanp, *xs2, xs2p;
  bool allperm=false; 
  /****************************************************************************/
  /*                                                                          */
  /* Init random number generator.                                            */
  /*                                                                          */
  /****************************************************************************/
  
  if (rand) initrseed(-time(0));
  else initrseed(0);

  /****************************************************************************/
  /*                                                                          */
  /* Calculate total sample number. Prepare multisample generation.           */
  /*                                                                          */
  /****************************************************************************/
  
  for (i=s; i>0; i--) n+=ns[i];
  dn=(double)n;
  if (multisample) {
    nl=lvector(1,s);
    nh=lvector(1,s);
    for (i=2, nl[1]=1, nh[1]=ns[1]; i<=s; i++) {
      nl[i]=nh[i-1]+1;
      nh[i]=nh[i-1]+ns[i];
    }
  }

  /****************************************************************************/
  /*                                                                          */
  /* Allocate memory for variable means and variances. Set pointers to last   */
  /* elements. Reduce 'p' by 1 in order to exclude the seed variable.         */
  /*                                                                          */
  /****************************************************************************/
  
  xmean=dvector(1,p);
  xs2=dvector(1,p);
  x2s=dvector(1,p);

  X+=n*p;
  xmean+=p;
  xs2+=p;
  x2s+=p;

  for (i=p; i>0; i--) {
    idx[i]=i;
    for (j=n, sum=0, sum2=0; j>0; j--) {
      x=*X;
      sum+=x;
      sum2+=x*x;
      X--;
    }
    *xmean=sum/dn;
    *(xs2--)=sqrt(sum2-dn*(*xmean)*(*xmean));
    *(x2s--)=sum2;
    xmean--;
  }

  /****************************************************************************/
  /*                                                                          */
  /* Loop over all variables and calculate correlation between the pivot      */
  /* variable and all other variables. Count correlations above correlation   */
  /* threshold and store index of correlated variables in 'clustidx'.         */
  /* Note: Without the seed variable there are only 'p'-1 variables.          */
  /*                                                                          */
  /****************************************************************************/

  clustlen=lvector(1,p-1);
  clenoffset=lvector(1,p-1);
  clustidx=lvector(1,p-1);

  for (i=1, clens=0; i<p; i++) {
    clenoffset[i]=clens;
    for (j=1; j<p; j++) {
      
      /************************************************************************/
      /*                                                                      */
      /* Dynamically enlarge the allocated memory for clustidx. The           */
      /* enlargement is for 'p'-1 indices. So there may be a memory overhang  */
      /* that has to be corrected at the end of cluster generation.           */
      /*                                                                      */
      /************************************************************************/
 
      if (!(clens%(p-1))) {
        clustidx++;
        clustidx=(int*)realloc(clustidx,(size_t)(sizeof(int)*(clens+p-1)));
        clustidx--;
      }

      if (i==j) {
	clustidx[++clens]=j;
	continue;
      }

      /************************************************************************/
      /*                                                                      */
      /* If variance dominance of the pivot variable is requested. The        */
      /* variances of all the other variables are inspected first. If the     */
      /* pivot variable has a variance less than the other variable the       */
      /* current loop is skipped.                                             */
      /*                                                                      */
      /************************************************************************/

      if (pivdom) if (x2s[i]<x2s[j]) continue;
      
      xi=X+i*n;
      xj=X+j*n;
      
      /************************************************************************/
      /*                                                                      */
      /* Sum up xi*yi.                                                        */
      /*                                                                      */
      /************************************************************************/
      
      for (k=n, sum=0.; k>0; k--) sum+=*(xi--)*(*(xj--));
      
      /************************************************************************/
      /*                                                                      */
      /* Calculate the PEARSON correlation coefficient.                       */
      /*                                                                      */
      /************************************************************************/
      
      r=(sum-dn*xmean[i]*xmean[j])/(xs2[i]*xs2[j]);
      
      /************************************************************************/
      /*                                                                      */
      /* If only positive correlations should be taken into account 'r' is    */
      /* compared with zero. If, in case of 'rposonly' equals 'TRUE', the     */
      /* correlation coefficient is negative the current loop is skipped.     */
      /*                                                                      */
      /************************************************************************/

      if (rposonly) if (r<0) continue;
      
      if (r*r>r2min) clustidx[++clens]=j;
    }
    clustlen[i]=clens-clenoffset[i];
  }

  /****************************************************************************/
  /*                                                                          */
  /* Fit cluster variable memory allocation.                                  */
  /*                                                                          */
  /****************************************************************************/

  clustidx++;
  clustidx=(int*)realloc(clustidx,(size_t)(sizeof(int)*clens));
  clustidx--;

  /****************************************************************************/
  /*                                                                          */
  /* Calculate the test values for all groups, that is the sum of the         */
  /* correlation coefficients of the variables of a cluster with the seed     */
  /* variable.                                                                */
  /*                                                                          */
  /****************************************************************************/

  xmeanp=xmean[p];
  xs2p=xs2[p];

  for (i=p-1; i>0; i--) {
    for (j=clustlen[i],sum2=0.; j>0; j--) {
      xp=X+n*p;
      xj=X+n*clustidx[clenoffset[i]+j];
      for (k=n, sum=0.; k>0; k--) sum+=*(xp--)*(*(xj--));
      r=(sum-dn*xmeanp*xmean[clustidx[clenoffset[i]+j]])/
	(xs2p*xs2[clustidx[clenoffset[i]+j]]);
      sum2+=r*r;
    }
    testvalue[i]=sum2;
  }

  /****************************************************************************/
  /*                                                                          */
  /* Sort the testvalues in increasing order. With-sort the index vector to   */
  /* keep track of the old variable position.                                 */
  /*                                                                          */
  /****************************************************************************/
 
  dlqsort2(testvalue,idx,1,p-1);

  /****************************************************************************/
  /*                                                                          */
  /* if nres==0 then all possible permutations should be generated.           */
  /*                                                                          */
  /****************************************************************************/
  
  if (nres==0) {
      if (n>MAXFAC) {
          mathwarning("too many samples (n=12) for generating \
                       all permutations. Continuing with 1000 ramdom\
                       permutations.");
      nres=1000;
      dnres=1000.0;
      } else {
          nres=fac(n);
          dnres=(double)nres;
          allperm=true;
      } 
  }

  /****************************************************************************/
  /*                                                                          */
  /* The data structure to yield the permutation index vectors is a vector of */
  /* pointers to vectors. Each vector contains a permutation of the n indices */
  /* 1...n.                                                                   */
  /* The type of such a construct is "int **" (pointer to a vector of        */
  /* pointers to "int". As each element of "permidx" is a pointer to a       */
  /* vector of "int" values they have to be allocated separately.            */
  /*                                                                          */
  /****************************************************************************/

  permidx=(int **)malloc((size_t)(sizeof(int *)*(nres-1)));
  permidx-=2;
  
  if (allperm) {   // all permutations

  /****************************************************************************/
  /* init the permutation index vector with 1,..,n                            */
  /****************************************************************************/
 
      int* tmpidx=lvector(1,n);              
      for (i=1; i<=n; i++) tmpidx[i]=i;   

  /****************************************************************************/
  /* generate next permutation of the index vector and copy it to permidx[h]  */
  /****************************************************************************/
     
      for (h=2; h<=nres; h++) {
           permidx[h]=lvector(1,n);
           lallsample(tmpidx,1,n);
           for (i=1; i<=n; i++) permidx[h][i]=tmpidx[i];
      }
      free_lvector(tmpidx,1);
  } else         // random permutations
      for (h=2; h<=nres; h++) {
        permidx[h]=lvector(1,n);
        if (multisample) lmultisample(permidx[h],nl,nh,ns,s,FALSE); 
        else lsample(permidx[h],1,n,n,FALSE);
      }

  /****************************************************************************/
  /*                                                                          */
  /* Allocate and initialize distribution vector                              */
  /*                                                                          */
  /****************************************************************************/

  dist=dvector(1,nres);
  for (h=nres; h>0; h--) dist[h]=0.;

  /****************************************************************************/
  /*                                                                          */
  /* The following lines do the W/Y resampling. The process starts with the   */
  /* "worst" variable. For that variable all resampling variants are realized */
  /* and for each variant the test value is calculated. The test values are   */
  /* stored in the 'dist'-variable. At last the P-value of the variable is    */
  /* calculated using the distribution stored in 'dist'.                      */
  /* Then the next best variable is taken and subdued to the same process     */
  /* (the resampling order has to be the same) and for each resampling data   */
  /* the 'dist'-variable is updated if necessary (that means a larger test    */
  /* value has been found).                                                   */
  /*                                                                          */
  /****************************************************************************/
  
  fprintf(stdout,"\n");
  fflush(stdout);
  xp=X+n*(p-1);

  for (i=1; i<=p-1; i++) {

    /**************************************************************************/
    /*                                                                        */
    /* Get gene index of the original (unsorted) order get appropriate sum of */
    /* the values of that gene.                                               */
    /*                                                                        */
    /**************************************************************************/

    idxi=idx[i];
    clenoffsetidxi=clenoffset[idxi];
    fprintf(stdout,"\rProgress: %6.2lf %%",(double)i/(double)p*100.);
    fflush(stdout);

    /**************************************************************************/
    /*                                                                        */
    /* Set 'pcount' to one. That is possible because the original test values */
    /* beint to the resampling distribution. So there is at least one value  */
    /* greater than or equal to the test value.                               */
    /* If the test value of the current gene is better than the appropriate   */
    /* value of the distribution the distribution variable 'dist' is updated. */
    /*                                                                        */
    /**************************************************************************/

    pcount=1;
    if (dist[1]<testvalue[i]) dist[1]=testvalue[i];

    /**************************************************************************/
    /*                                                                        */
    /* Loop through all resamplings.                                          */
    /*                                                                        */
    /**************************************************************************/
    
    for (h=2; h<=nres; h++) {
      
      /************************************************************************/
      /*                                                                      */
      /* Get permutation index for the current resampling.                    */
      /*                                                                      */
      /************************************************************************/
     
      permidxh=permidx[h];
      for (j=clustlen[idxi], test=0.; j>0; j--) {
	xj=X+n*(clustidx[clenoffsetidxi+j]-1);
        clustidxj=clustidx[clenoffsetidxi+j];
	for (k=n, sum=0.; k>0; k--) sum+=xp[permidxh[k]]*xj[k];
	r=(sum-dn*xmeanp*xmean[clustidxj])/(xs2p*xs2[clustidxj]);
	test+=r*r;
      }

      /************************************************************************/
      /*                                                                      */
      /* If the test value is greater than the maximum of the test values of  */
      /* former variables the distribution is updated.                        */
      /*                                                                      */
      /************************************************************************/

      if (dist[h]<test) dist[h]=test;
      
      /************************************************************************/
      /*                                                                      */
      /* If the original test value of the current variable is less than or   */
      /* equal to the distribution value of the current resampling 'pcount'   */
      /* is incremented. (With 'pcount' the empiric P-value is calculated.)   */
      /*                                                                      */
      /************************************************************************/

      if (testvalue[i]<=dist[h]) pcount++;
    }

    pvalue[i]=(double)pcount/dnres;

  }

  /****************************************************************************/
  /*                                                                          */
  /* Adjust P-values for monotony so that P[i+1] <= P[i].                     */
  /*                                                                          */
  /****************************************************************************/

  for (i=p-2; i>0; i--)
    if (pvalue[i+1] > pvalue[i]) pvalue[i]=pvalue[i+1];

  /****************************************************************************/
  /*                                                                          */
  /* Calculate critical values.                                               */
  /*                                                                          */
  /****************************************************************************/

  dqsort(dist,1,nres);

  for (i=1; i<=ncrit; i++) {
    critvals[i]=dist[(int)floor(dnres*(1.-critlevs[i]))];
  }

  /****************************************************************************/
  /*                                                                          */
  /* Free locally allocated memory.                                           */
  /*                                                                          */
  /****************************************************************************/
  
  free_dvector(dist,1);
  free_dvector(xmean,1);
  free_dvector(xs2,1);
  free_dvector(x2s,1);

  for (h=2; h<=nres;h++) free_lvector(permidx[h],1);
  free(permidx+2);
  if (multisample) {
    free_lvector(nl,1);
    free_lvector(nh,1);
  }
  free_lvector(clenoffset,1);
  free_lvector(clustidx,1);
  free_lvector(clustlen,1);

  fprintf(stdout,"\n");
  fflush(stdout);
}
