/******************************************************************************/
/*                                                                            */
/* wycorrlaeuter.[ch]                                                         */
/* ==================                                                         */
/*                                                                            */
/* Implementation of the procedure by Juergen Laeuter for the search of gene  */
/* groups highly correlated to a given seed genes in a (multiclass)           */
/* microarray data set.                                                       */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Standard header files.                                                     */
/*                                                                            */
/******************************************************************************/

#ifndef WYCORRLAEUTER_H
#define WYCORRLAEUTER_H

#include "mathutils.h"
#include "computebetasum.h"
//#include "trimsets.h"

/******************************************************************************/
/*                                                                            */
/* Functions.                                                                 */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* A Westfall/Young procedure implementing the screening process described by */
/* Juergen Laeuter. In a first step all groups of correlated variables are    */
/* computed. For all those groups the sum of correlation coefficients of the  */
/* variables in a group with a seed variable are calculated. The sums are     */
/* taken as test values for the similarity of the groups and the seed         */
/* variable. The test values are subjected to a Westfall & Young resampling   */
/* procedure in order to correct for multiple testing.                        */
/*                                                                            */
/* This function uses PEARSON correlation for group building.                 */
/*                                                                            */
/* Function parameters:                                                       */
/*                                                                            */
/* X - a data matrix represented as vector of length 'n' x 'p' that is a      */
/*     concatenation of the 'p' column vectors of length 'n' of an original   */
/*     (n x p)-matrix. The seed variable is in the last column of 'X'.        */
/*                                                                            */
/* ns - the numbers of samples in the sample groups.                          */
/*                                                                            */
/* s - the number of sample groups.                                           */
/*                                                                            */
/* p - the number of variable.                                                */
/*                                                                            */
/* nres - the number of resamplings that is determined inside the function.   */
/*                                                                            */
/* rand - a boolean variable deciding whether to seed the random number       */
/*        generator or not.                                                   */
/*                                                                            */
/* multisample - a boolean variable indicating whether the permutations       */
/*               should be restricted to the sample groups or not.            */
/*                                                                            */
/* rot - a boolean switch. If TRUE then the resampling is done with rotations */
/*       otherwise it is done with permutations                               */
/*                                                                            */
/* rposonly - a boolean switch. If 'rposonly' is 'TRUE' only positive         */
/*            correlations are taken into account.                            */
/*                                                                            */
/* pivdom - a boolean switch. If 'pivdom' is 'TRUE' variance of the pivot     */
/*          variable is enforced. That suppresses symmetric group overhead.   */
/*                                                                            */
/* r2min - a threshold for the square of the correlation coefficient.         */
/*                                                                            */
/* alpha - a parameter to constrain the variable space.                       */
/*                                                                            */
/* clustidx - a vector containing the indices of variables clustered.         */
/*                                                                            */
/* clustlen - the numbers of variables in the clusters.                       */
/*                                                                            */
/* idx - As part of the W/Y procedure the variables are sorted by their test  */
/*       values. In order to keep track of the variables the new index of any */
/*       variable is stored at its old position in 'idx'.                     */
/*                                                                            */
/* testvalue - the values of the test function (sorted).                      */
/*                                                                            */
/* pvalue - the P-value of each test value obtained from the W/Y resampling   */
/*          distribution.                                                     */
/*                                                                            */
/* direction - equal to 1 if every pivot variable is required to be           */ 
/*             positively correlated with the target variable                 */ 
/*             (one-sided test, only positive significance)                   */
/*             equal to -1 if every pivot variable is required to be          */ 
/*             negatively correlated with the target variable                 */
/*             (one-sided test, only negatibe significance)                   */
/*             equal to 0 if there are no such requirements                   */
/*             (two-sided test)                                               */
/* Return value(s):                                                           */
/*                                                                            */
/* All results are returned implicitely using the references of 'idx',        */
/* 'testvalue' and 'pvalue'.                                                  */
/*                                                                            */
/******************************************************************************/

void wy_corr_laeuter_pearson(double *X, int *ns, int s, int p,
                             int *grpidx, int *glen, int gnum, 
                             int *pivotidx, bool rposonly,
			     int direction, bool pivdom, double r2min,
			     int nres, bool rand, bool multisample, bool rot,
                             bool discret, bool trunc, double ucrit, 
                             double minbeta, bool samedirect, int maxgene,
                             bool singlestep, 
			     int *idx, double *testvalue, double *pvalue,
                             double *critlevs, int ncrit, double *critvals);

/******************************************************************************/
/*                                                                            */
/* A Westfall/Young procedure implementing the screening process described by */
/* Juergen Laeuter. In a first step all groups of correlated variables are    */
/* computed. For all those groups the sum of correlation coefficients of the  */
/* variables in a group with a seed variable are calculated. The sums are     */
/* taken as test values for the similarity of the groups and the seed         */
/* variable. The test values are subjected to a Westfall & Young resampling   */
/* procedure in order to correct for multiple testing.                        */
/*                                                                            */
/* This function uses total correlation for group building.                   */
/*                                                                            */
/* Function parameters:                                                       */
/*                                                                            */
/* X - a data matrix represented as vector of length 'n' x 'p' that is a      */
/*     concatenation of the 'p' column vectors of length 'n' of an original   */
/*     (n x p)-matrix. The seed variable is in the last column of 'X'.        */
/*                                                                            */
/* ns - the numbers of samples in the sample groups.                          */
/*                                                                            */
/* s - the number of sample groups.                                           */
/*                                                                            */
/* p - the number of variable.                                                */
/*                                                                            */
/* nres - the number of resamplings that is determined inside the function.   */
/*                                                                            */
/* rand - a boolean variable deciding whether to seed the random number       */
/*        generator or not.                                                   */
/*                                                                            */
/* multisample - a boolean variable indicating whether the permutations       */
/*               should be restricted to the sample groups or not.            */
/*                                                                            */
/* rposonly - a boolean switch. If 'rposonly' is 'TRUE' only positive         */
/*            correlations are taken into account.                            */
/*                                                                            */
/* pivdom - a boolean switch. If 'pivdom' is 'TRUE' variance of the pivot     */
/*          variable is enforced. That suppresses symmetric group overhead.   */
/*                                                                            */
/* r2min - a threshold for the square of the correlation coefficient.         */
/*                                                                            */
/* alpha - a parameter to constrain the variable space.                       */
/*                                                                            */
/* clustidx - a vector containing the indices of variables clustered.         */
/*                                                                            */
/* clustlen - the numbers of variables in the clusters.                       */
/*                                                                            */
/* idx - As part of the W/Y procedure the variables are sorted by their test  */
/*       values. In order to keep track of the variables the new index of any */
/*       variable is stored at its old position in 'idx'.                     */
/*                                                                            */
/* testvalue - the values of the test function (sorted).                      */
/*                                                                            */
/* pvalue - the P-value of each test value obtained from the W/Y resampling   */
/*          distribution.                                                     */
/*                                                                            */
/* Return value(s):                                                           */
/*                                                                            */
/* All results are returned implicitely using the references of 'idx',        */
/* 'testvalue' and 'pvalue'.                                                  */
/*                                                                            */
/******************************************************************************/

void wy_corr_laeuter_total(double *X, int *ns, int s, int p, bool rposonly,
			   bool pivdom, double r2min,
			   int nres, bool rand, bool multisample, 
			   int *idx, double *testvalue, double *pvalue,
                           double *critlevs, int ncrit, double *critvals);

#endif
