/******************************************************************************/
/*                                                                            */
/* wyutils.c / wyutils.h                                                      */
/* =====================                                                      */
/*                                                                            */
/* Utility functions that support the Westfall & Young resampling algorithms  */
/* implemented in the Westfall & Young-package.                               */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Standard header files.                                                     */
/*                                                                            */
/******************************************************************************/

#include <math.h>

/******************************************************************************/
/*                                                                            */
/* User header files.                                                         */
/*                                                                            */
/******************************************************************************/

#include "mathutils.h"
#include "wyutils.h"

/******************************************************************************/
/*                                                                            */
/* Calculate number of systematic commutations.                               */
/*                                                                            */
/******************************************************************************/

int getsystcommnum(int n) {
  return (int)pow(2.,(double)(n-1));
}
                                                                                
/******************************************************************************/
/*                                                                            */
/* Calculate number of systematic permutations (two groups).                  */
/*                                                                            */
/******************************************************************************/

int getsystpermnum(int *ns) {
  int b=1, i;
  for (i=1; i<=ns[1]; i++) {
    b*=(ns[2]+i);
    b/=i;
  }
                                                                                
  if (ns[1]==ns[2]) b/=2;
                                                                                
  return b;
}

/******************************************************************************/
/*                                                                            */
/* Compute in and out indices for the 'Revolving door algorithm' an algorithm */
/* for systematic generation of all k-combinations out of n elements by       */
/* exchanging only one element at a time.                                     */
/*                                                                            */
/******************************************************************************/

void getinoutidxs(int *ns, int *inidx, int *outidx, int permnum) {
  int b=permnum, i=1, j, n=ns[1]+ns[2], t=ns[1];
  int nextrule=3;
  int *c;

  /* R1 */
  c=lvector(1,t+1);
  for (j=1; j<=t; j++) c[j]=j-1;
  c[t+1]=n;
  i=2;
  
  while (i <= b) {

    /* R3 */
    if (nextrule==3) {
      if ((t & 1) == 1) {
        /* t is odd */
        if (c[1]+1 < c[2]) {
          outidx[i]=c[1]+1;
          c[1]++;
          inidx[i]=c[1]+1;
	  i++;
          continue;
        } else {
          j=2;
          nextrule=4;
        }
      } else {
        /* t is even */
        if (c[1] > 0) {
          outidx[i]=c[1]+1;
          c[1]--;
          inidx[i]=c[1]+1;
	  i++;
          continue;
        } else {
          j=2;
          nextrule=5;
        }
      }
    }

    /* R4 */
    if (nextrule==4) {
      if (c[j] >= j) {
        outidx[i]=c[j]+1;
        c[j]=c[j-1];
        c[j-1]=j-2;
        inidx[i]=c[j-1]+1;
        nextrule=3;
	i++;
        continue;
      } else {
        j++;
        nextrule=5;
      }
    }

    /* R5 */
    if (nextrule==5) {
      if (c[j]+1 < c[j+1]) {
        outidx[i]=c[j-1]+1;
        c[j-1]=c[j];
        c[j]++;
        inidx[i]=c[j]+1;
        nextrule=3;
	i++;
        continue;
      } else {
        j++;
        if (j<=t) {
          nextrule=4;
          continue;
        } else {
          nextrule=3;
	  i++;
          continue;
        }
      }
    }
  }
  
  free_lvector(c,1);
}  
